Package: SurfR
Type: Package
Title: Surface Protein Prediction and Identification
Version: 1.8.0
Authors@R: c(person("Aurora", "Maurizio", email="auroramaurizio1@gmail.com", role = c("aut","cre"), comment = c(ORCID = "0000-0002-7194-4637")),
           person("Anna Sofia", "Tascini", email="volpesofi@gmail.com", role = c("aut", "ctb"), comment = c(ORCID = "0000-0001-5731-5490")))
Description: Identify Surface Protein coding genes from a list of candidates.
             Systematically download data from GEO and TCGA or use your own data. 
             Perform DGE on bulk RNAseq data. 
             Perform Meta-analysis. Descriptive enrichment analysis and plots.
License: GPL-3 + file LICENSE
Encoding: UTF-8
LazyData: false
BugReports: https://github.com/auroramaurizio/SurfR/issues
URL: https://github.com/auroramaurizio/SurfR
biocViews: Software, Sequencing, RNASeq, GeneExpression, Transcription,
        DifferentialExpression, PrincipalComponent, GeneSetEnrichment,
        Pathways, BatchEffect, FunctionalGenomics, Visualization,
        DataImport, FunctionalPrediction, GenePrediction, GO
VignetteBuilder: knitr
RoxygenNote: 7.2.3
Imports: httr, BiocFileCache, SPsimSeq, DESeq2, edgeR, openxlsx,
        stringr, rhdf5, ggplot2, ggrepel, stats, magrittr, assertr,
        tidyr, dplyr, TCGAbiolinks, biomaRt, metaRNASeq, scales, venn,
        gridExtra, SummarizedExperiment, knitr, rjson, grDevices,
        graphics, curl, utils
Depends: R (>= 4.4.0)
Suggests: BiocStyle, testthat (>= 3.0.0)
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/SurfR
git_branch: RELEASE_3_23
git_last_commit: fba3adc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:41:34 UTC; biocbuild
Author: Aurora Maurizio [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7194-4637>),
  Anna Sofia Tascini [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-5731-5490>)
Maintainer: Aurora Maurizio <auroramaurizio1@gmail.com>
Built: R 4.6.0; ; 2026-04-29 05:03:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Annotate_SPID)
export(DGE)
export(DownloadArchS4)
export(Enrichment)
export(Enrichment_barplot)
export(GEOmetadata)
export(Gene2SProtein)
export(SVenn)
export(Splot)
export(TCGA_download)
export(combine_fisher_invnorm)
export(enrichr)
export(enrichr_download)
export(listEnrichrDbs)
export(metaRNAseq)
export(plotPCA)
export(setEnrichrSite)
import(BiocFileCache)
import(SPsimSeq)
import(biomaRt)
import(knitr)
importFrom(DESeq2,DESeq)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,counts)
importFrom(DESeq2,results)
importFrom(SummarizedExperiment,assay)
importFrom(TCGAbiolinks,GDCdownload)
importFrom(TCGAbiolinks,GDCprepare)
importFrom(TCGAbiolinks,GDCquery)
importFrom(assertr,col_concat)
importFrom(dplyr,group_by)
importFrom(dplyr,relocate)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(edgeR,cpm)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_label_repel)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,hist)
importFrom(gridExtra,marrangeGrob)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,content)
importFrom(httr,http_status)
importFrom(httr,status_code)
importFrom(httr,use_proxy)
importFrom(magrittr,"%>%")
importFrom(metaRNASeq,fishercomb)
importFrom(metaRNASeq,invnorm)
importFrom(openxlsx,read.xlsx)
importFrom(openxlsx,write.xlsx)
importFrom(rhdf5,H5close)
importFrom(rhdf5,h5read)
importFrom(rjson,fromJSON)
importFrom(scales,hue_pal)
importFrom(stats,as.formula)
importFrom(stats,prcomp)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(tidyr,separate_rows)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(venn,venn)

# SurfR

Proteins at the cell surface connect intracellular and extracellular
 signaling networks and largely determine a cell’s capacity to 
communicate and interact with its environment. 

Importantly, variations in transcriptomic profiles are often observed
between healthy and diseased cells, presenting distinct sets 
of cell-surface proteins. Indeed, cell surface proteins 
i) may act as biomarkers for the detection of diseased cells
 in tissues or body fluids 
and 
ii) are the most prevalent target of pharmaceutical agents:
 66% of approved human drugs listed in the DrugBank database 
target a cell-surface protein. 
The investigation of the cell surfaceome therefore could 
provide new possibilities for diagnosis, prognosis, 
treatment development, and therapy response evaluation.



## What is SurfR

The **SurfR** package aims to provide a streamlined end-to-end workflow for 
identifying surface protein coding genes from expression data using computational prediction.



**SurfR** :

-   Returns a list of of surface protein coding genes, starting from 
    a list of genes of interest, the raw count matrix of your own
    RNA-seq experiment, or from bulk transcriptomic data 
    automatically retrieved from public databases. Protein classification 
    is based on a recently developed surfaceome predictor, 
    called SURFY, based on machine learning. 
-   Allows automatic data retrieval from public databases such as 
    GEO and TCGA. GEO queries are based on the ArchS4 pipeline. 
    TCGA repository is interrogated through TCGAbiolinks.
-   Provides a function for differential gene expression analysis. 
    For this task it relies on DESeq2 package, starting from counts data. 
-   Offers the opportunity to increase the sample size of a cohort
    by integrating related datasets, therefore enhancing the power
    to detect differentially expressed genes of interest. 
    Meta-analysis can be performed through metaRNASeq, taking into
    account inter-study variability that may arise from technical
    differences among studies (e.g., sample preparation, library
    protocols, batch effects) as well as additional biological
    variability.
-   Gene ontology (GO) and pathway annotation can also be performed
    within **SurfR** to gain further insights about surface protein
    candidates.
-   Includes functions to visualize DEG and enrichment results,
    including BarPlots, Histograms, Venn diagrams, and PCA plots to help 
    users achieve efficient data interpretation.



## Installation

To install this package, start R (version "4.4") and enter:

```{r install, eval = FALSE}

if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
```

When the package is available on *Bioconductor*, use

```{r install-Bioconductor, eval = FALSE}
BiocManager::install("SurfR")
```

Development package version can be installed from GitHub using devtools:
```
devtools::install_github("auroramaurizio/SurfR")
```


## Dependencies
This package is supported for macOS, and Linux (Windows not tested). 
**SurfR** works with R v4.4 or greater.
Dependencies are indicated in the DESCRIPTION file, and can be 
automatically installed when installing the **SurfR** pacakge. 

## Vignettes

A comprehensive vignette provides an introduction to the **SurfR** package. 
Examples and use-cases are covered for each function.
Additional RMD notebooks containing the use cases code described in the manuscript are available on GitHub: https://github.com/auroramaurizio/SurfR_UseCases.

## Documentation

Instructions to run the main functions can be found consulting the vignette
 or by entering ?FunctionName (e.g. ?Splot) in the console after loading the package.

## Citation

Maurizio, A., Tascini, A.S., Morelli, M. SurfR: Riding the wave of RNA-Seq data with a comprehensive Bioconductor package to identify Surface Protein Coding Genes. Bioinformatics Advances, 2024 (DOI: 10.1093/bioadv/vbae201)

## Authors

Aurora Maurizio (auroramaurizio1@gmail.com), 
Anna Sofia Tascini (volpesofi@gmail.com), 
Marco Morelli (morelli.marco@hsr.it)


## Help, Suggestions, and Contributions

Any contribution is highly appreciated! 
If you are interested in contributing to this project, please open an issue.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----install, eval = FALSE----------------------------------------------------
# if (!"BiocManager" %in% rownames(installed.packages()))
#     install.packages("BiocManager", repos = "https://cran.r-project.org")

## ----install-Bioconductor, eval = FALSE---------------------------------------
# BiocManager::install("SurfR")

## ----install-github, eval = FALSE---------------------------------------------
# # install.packages("devtools")
# devtools::install_github("auroramaurizio/SurfR", build_vignettes = TRUE)

## ----gene2protein, eval=FALSE-------------------------------------------------
# library(SurfR)
# 
# GeneNames <- c("CIITA", "EPCAM", "CD24", "CDCP1", "LYVE1")
# SurfaceProteins_df <- Gene2SProtein(GeneNames,
#                                     input_type = "gene_name",
#                                     output_tsv = FALSE,
#                                     Surfy_version = "new")
# #The output dataframe contains several information retrieved from Surfy.
# colnames(SurfaceProteins_df)
# #These are the 5 surface protein coding genes detected by SurfR.
# SurfaceProteins_df$UniProt.gene

## ----simulate zhang, eval=FALSE-----------------------------------------------
# library(SPsimSeq)
# 
# data("zhang.data.sub")
# zhang.counts <- zhang.data.sub$counts
# MYCN.status  <- zhang.data.sub$MYCN.status
# 
# # Simulation of bulk RNA data
# sim.data.bulk <- SPsimSeq(n.sim = 1,
#                           s.data = zhang.counts,
#                           group = MYCN.status,
#                           n.genes = 1000,
#                           batch.config = 1,
#                           group.config = c(0.5, 0.5),
#                           tot.samples = ncol(zhang.counts),
#                           pDE = 0.2,
#                           lfc.thrld = 0.5,
#                           result.format = "list",
#                           return.details = TRUE)
# 
# sim.data.bulk1 <- sim.data.bulk$sim.data.list[[1]]
# 
# countMatrix <- sim.data.bulk1$counts
# row.names(countMatrix) <- row.names(zhang.counts)
# metadata <- sim.data.bulk1$colData
# metadata$Group <- as.factor(metadata$Group)

## ----DGE zhang, eval=FALSE----------------------------------------------------
# library(SurfR)
# 
# df_zhang <- DGE(expression = countMatrix,
#                 metadata = metadata,
#                 Nreplica = 50,
#                 design = "~Group",
#                 condition = "Group",
#                 alpha = 0.05,
#                 TEST = "1", CTRL =  "0",
#                 output_tsv = FALSE)
# 
# head(df_zhang)

## ----G2P zhang, eval=FALSE----------------------------------------------------
# # remove NA values
# df_zhang <- df_zhang[!is.na(df_zhang$padj), ]
# 
# # all fdr
# fdr_GeneID <- df_zhang[df_zhang$padj < 0.05, "GeneID"]
# SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")
# 
# # upregulated fdr
# fdrUP_GeneID <- df_zhang[df_zhang$padj < 0.05 & df_zhang$log2FoldChange > 0,
#                          "GeneID"]
# SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")
# 
# # dowregulated fdr
# fdrDW_GeneID <- df_zhang[df_zhang$padj < 0.05 & df_zhang$log2FoldChange < 0,
#                          "GeneID"]
# SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")

## ----GEO input meta_counts, eval=FALSE----------------------------------------
# library(SurfR)
# library(stringr)
# 
# # Download metadata from GEO
# mGSE121810 <- GEOmetadata(GSE = "GSE121810")
# 
# # create new metadata column in order to remove unwanted special characters
# unwanted_character <- " "
# fx <- function(x) {
#   str_split(string = x, pattern = unwanted_character)[[1]][1]
# }
# mGSE121810$condition <- sapply(mGSE121810$therapy, fx)
# mGSE121810$condition <- as.factor(mGSE121810$condition)
# 
# # Preview metadata
# head(mGSE121810)
# 
# # only select 3 samples per condition to save time
# na_samples <- c("GSM3447013", "GSM3447018", "GSM3447019")
# a_samples <- c("GSM3447023", "GSM3447024", "GSM3447026")
# mGSE121810 <- mGSE121810[c(na_samples, a_samples), ]
# 
# # Download count matrix from ArchS4
# cGSE121810 <- DownloadArchS4(mGSE121810$GSM,
#                              species = "human",
#                              print_tsv = FALSE,
#                              filename = NULL)
# 
# # Preview count matrix
# head(cGSE121810[, ])

## ----GEO dge, eval=FALSE------------------------------------------------------
# # Perform DGE
# df_GEO <- DGE(expression = cGSE121810,
#               metadata = mGSE121810,
#               Nreplica = 3,
#               design = "~condition",
#               condition = "condition",
#               alpha = 0.05,
#               TEST = "neoadjuvant", CTRL =  "adjuvant",
#               output_tsv = FALSE)
# 
# # remove NA values
# df_GEO <- df_GEO[!is.na(df_GEO$padj), ]
# 
# head(df_GEO)

## ----GEO SP, eval=FALSE-------------------------------------------------------
# # Detect SP amoung differentially expressed genes
# fdr_GeneID <- df_GEO[df_GEO$padj < 0.1, "GeneID"]
# 
# SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")
# 
# fdrUP_GeneID <- df_GEO[df_GEO$padj < 0.1 & df_GEO$log2FoldChange > 0, "GeneID"]
# SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")
# 
# fdrDW_GeneID <- df_GEO[df_GEO$padj < 0.1 & df_GEO$log2FoldChange < 0, "GeneID"]
# SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")
# 

## ----TCGA install, eval=FALSE-------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("BioinformaticsFMRP/TCGAbiolinksGUI.data")
# BiocManager::install("BioinformaticsFMRP/TCGAbiolinks")

## ----TCGA THYM Download, warning=FALSE, message = FALSE, eval=FALSE-----------
# # Download TCGA data
# # To save time only download 3 Tumor samples and 3 normal samples.
# # If barcodes are not specified TCGA_download function will download all the
# # available samples in the TCGA study
# barcodes2Download <- c("TCGA-X7-A8D6-11A-22R-A42C-07",
#                        "TCGA-X7-A8D7-11A-11R-A42C-07",
#                        "TCGA-XM-A8RB-01A-11R-A42C-07",
#                        "TCGA-X7-A8M0-01A-11R-A42C-07")
# TCGA.THYM <- TCGA_download(project = "TCGA-THYM", barcodes = barcodes2Download)
# 
# cTCGA.THYM <- TCGA.THYM[[1]]
# mTCGA.THYM <- TCGA.THYM[[2]]
# table(mTCGA.THYM$shortLetterCode)
# 
# mTCGA.THYM$shortLetterCode <- as.factor(mTCGA.THYM$shortLetterCode)

## ----TCGA DGE, eval=FALSE-----------------------------------------------------
# df_TCGA <- DGE(expression = cTCGA.THYM,
#                metadata = mTCGA.THYM,
#                Nreplica = 2,
#                design = "~shortLetterCode",
#                condition = "shortLetterCode",
#                alpha = 0.05,
#                TEST = "TP", CTRL =  "NT",
#                output_tsv = FALSE)
# 
# head(df_TCGA)

## ----TCGA SP, eval=FALSE------------------------------------------------------
# # remove NA values
# df_TCGA <- df_TCGA[!is.na(df_TCGA$padj), ]
# 
# fdr_GeneID <- df_TCGA[df_TCGA$padj < 0.05,
#                       "GeneID"]
# 
# SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")
# 
# fdrUP_GeneID <- df_TCGA[df_TCGA$padj < 0.05 & df_TCGA$log2FoldChange > 0,
#                         "GeneID"]
# SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")
# 
# fdrDW_GeneID <- df_TCGA[df_TCGA$padj < 0.05 & df_TCGA$log2FoldChange < 0,
#                         "GeneID"]
# SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")

## ----TCGA BRCA Download, warning=FALSE, message = FALSE, eval=FALSE-----------
# # Download TCGA data
# # To save time only download 3 Tumor samples and 3 normal samples.
# # If barcodes are not specified TCGA_download function will download all the
# # available samples in the TCGA study
# barcodes2Download <- c("TCGA-BH-A1FU-11A-23R-A14D-07",
#                        "TCGA-BH-A1FC-11A-32R-A13Q-07",
#                        "TCGA-BH-A0DO-11A-22R-A12D-07",
#                        "TCGA-B6-A0RH-01A-21R-A115-07",
#                        "TCGA-BH-A1FU-01A-11R-A14D-07",
#                        "TCGA-A1-A0SE-01A-11R-A084-07")
# TCGA.BRCA <- TCGA_download(project = "TCGA-BRCA", barcodes = barcodes2Download)

## ----meta TCGA_input, eval=FALSE----------------------------------------------
# cTCGA.BRCA <- TCGA.BRCA[[1]]
# mTCGA.BRCA <- TCGA.BRCA[[2]]
# table(mTCGA.BRCA$shortLetterCode)
# 

## ----meta GEO_input metadata, eval = FALSE------------------------------------
# mGSE58135 <- GEOmetadata("GSE58135")
# mGSE58135 <- mGSE58135[mGSE58135$tissue != "Breast Cancer Cell Line", ]
# mGSE58135$condition <- "NT"
# mGSE58135$condition[mGSE58135$tissue %in% c("ER+ Breast Cancer Primary Tumor",
#                                             "Triple Negative Breast Cancer Primary Tumor")] <- "TP"
# 
# # only select 3 samples per condition to save time
# TP_samples <- c("GSM1401694", "GSM1401717", "GSM1401729")
# NT_samples <- c("GSM1401799", "GSM1401813", "GSM1401814")
# mGSE58135 <- mGSE58135[c(TP_samples, NT_samples), ]

## ----meta GEO_input counts, eval = FALSE--------------------------------------
# cGSE58135 <- DownloadArchS4(mGSE58135$GSM, species = "human")
# cGSE58135 <- cGSE58135[, row.names(mGSE58135)]
# 
# table(mGSE58135$condition)
# 

## ----meta DGE TCGA, eval=FALSE------------------------------------------------
# # TCGA DGE
# df.TCGA <- DGE(expression = cTCGA.BRCA,
#                metadata = mTCGA.BRCA,
#                Nreplica = 3,
#                design = "~shortLetterCode",
#                condition = "shortLetterCode",
#                alpha = 0.05,
#                TEST = "TP", CTRL =  "NT",
#                output_tsv = FALSE)
# head(df.TCGA)

## ----meta DGE GEO, eval = FALSE-----------------------------------------------
# # GSE58135 DGE
# df.GSE58135 <- DGE(expression = cGSE58135,
#                    metadata = mGSE58135,
#                    Nreplica = 3,
#                    design = "~condition",
#                    condition = "condition",
#                    alpha = 0.05,
#                    TEST = "TP", CTRL =  "NT",
#                    output_tsv = FALSE)
# head(df.GSE58135)

## ----meta fishercomb, eval=FALSE----------------------------------------------
# 
# L_fishercomb <- metaRNAseq(ind_deg = list(TCGA.BRCA =  df.TCGA, GEO.GSE58135 = df.GSE58135),
#                            test_statistic = "fishercomb",
#                            BHth = 0.05,
#                            adjpval.t = 0.05)

## ----meta invnorm, eval=FALSE-------------------------------------------------
# L_invnorm <- metaRNAseq(ind_deg = list(TCGA.BRCA =  df.TCGA, GEO.GSE58135 = df.GSE58135),
#                         test_statistic = "invnorm",
#                         BHth = 0.05,
#                         adjpval.t = 0.05,
#                         nrep = c(102, 56))

## ----metacombine, eval=FALSE--------------------------------------------------
# metacomb <- combine_fisher_invnorm(ind_deg = list(TCGA.BRCA =  df.TCGA,
#                                                   GEO.GSE58135 = df.GSE58135),
#                                    invnorm = L_invnorm,
#                                    fishercomb = L_fishercomb,
#                                    adjpval = 0.05)
# 
# 
# metacomb_GeneID <- metacomb[metacomb$signFC != 0, "GeneID"]
# SP <- Gene2SProtein(genes = metacomb_GeneID, input_type = "gene_name")

## ----metacombine up, eval=FALSE-----------------------------------------------
# metacombUP_GeneID <- metacomb[metacomb$signFC == 1, "GeneID"]
# SPup <- Gene2SProtein(genes = metacombUP_GeneID, input_type = "gene_name")

## ----metacombine dw, eval=FALSE-----------------------------------------------
# metacombDW_GeneID <- metacomb[metacomb$signFC == -1, "GeneID"]
# SPdw <- Gene2SProtein(genes = metacombDW_GeneID, input_type = "gene_name")
# 

## ----enrich, eval=FALSE-------------------------------------------------------
# 
# dfList <- list(GEO = df.GSE58135,
#                TCGA = df.TCGA)
# 
# # Enrichment analysis
# Enrich <- Enrichment(dfList,
#                      enrich.databases = c("GO_Biological_Process_2021",
#                                           "KEGG_2021_Human"),
#                      p_adj = 0.05, logFC = 1)
# 
# head(Enrich$GEO$fdr_up$GO_Biological_Process_2021)

## ----fig.width=7, fig.height=2.5, eval=FALSE----------------------------------
# library(ggplot2)
# 
# # barplot of the top 5 upregulated pathways
# Enrichment_barplot(Enrich$GEO,
#                    enrich.databases <- c("GO_Biological_Process_2021",
#                                          "KEGG_2021_Human"),
#                    p_adj = 0.05,
#                    num_term = 5,
#                    cond = "UP")
# 
# # barplot of the top 5 downregulated pathways
# Enrichment_barplot(Enrich$GEO,
#                    enrich.databases <- c("GO_Biological_Process_2021",
#                                          "KEGG_2021_Human"),
#                    p_adj = 0.05,
#                    num_term = 5,
#                    cond = "DOWN")

## ----anno spid, eval=FALSE----------------------------------------------------
# annotated <- Annotate_SPID(df.GSE58135, "WikiPathway_2021_Human")
# head(annotated, 10)

## ----fig.height= 4.5, fig.width=7, eval = FALSE-------------------------------
# # upregulated genes in GEO dataset
# fdrUP_GeneID <- df.GSE58135[df.GSE58135$padj < 0.05 & df.GSE58135$log2FoldChange > 0, "GeneID"]
# # corresponding Surface Proteins
# SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")
# # Barplot of Almen classification
# Splot(SPup,
#       group.by = "Membranome.Almen.main-class",
#       main = "Almen class")

## ----fig.height= 6, fig.width=7, eval = FALSE---------------------------------
# 
# S_list <- list(SP_1 = c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1"),
#                SP_2 = c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"))
# 
# SVenn(S_list,
#       cols.use = c("green", "blue"),
#       opacity = 0.5,
#       output_intersectionFile = FALSE)
# 

## ----fig.height= 6, fig.width=7, eval = FALSE---------------------------------
# 
# SurfR::plotPCA(matrix = edgeR::cpm(cGSE58135), metadata = mGSE58135,
#                dims = c(1, 2),
#                color.by = "condition", shape.by = "condition",
#                label = FALSE, main = "PCA GSE58135")

## -----------------------------------------------------------------------------
sessionInfo()

#' metaRNAseq function
#'
#' Perform Meta-Analysis of RNA-Seq Data
#'
#' @param ind_deg List of indipendent named DEG dataframes with p-values to be combined.
#' @param test_statistic p-value combination technique (inverse normal or Fisher):
#' \code{fishercomb}, \code{invnorm}.
#' By default: \code{fishercomb}.
#' @param BHth Benjamini Hochberg threshold.
#' @param nrep Vector of numbers of replicates used in each study to calculate the previous one-sided p-values.
#' @param adjpval.t threshold to represent as binary the Meta-Analysis output adjpval.
#' @param plot Logical. If TRUE plot histogram of pvalues.
#' By default, the False Discovery Rate is controlled at 0.05.
#' @return A list with \code{DEindices} of DEG at the chosen Benjamini Hochberg threshold, and
#' \code{TestStatistic}, \code{rawpval}, \code{adjpval}, \code{binaryadjpval} vectors for differential expression in the meta-analysis.
#' @examples
#' \dontrun{
#' # Deseq2 output samples
#' DGE1 <- data.frame(GeneID = c("DLK1", "EPCAM"),
#'                  Mean_CPM_T = c(5.92, 9.91),
#'                  Mean_CPM_C = c(0.04, 0.03),
#'                  log2FoldChange = c(10.22, 8.42),
#'                  lfcSE = c(0.80, 0.48),
#'                  stat = c(12.68, 17.69),
#'                  pvalue = c(7.30135e-37, 4.37011e-70),
#'                  padj = c(1.49936e-35, 1.12976e-67),
#'                  row.names = c("DLK1", "EPCAM"))
#' DGE2 <- data.frame(GeneID = c("DLK1", "EPCAM"),
#'                  Mean_CPM_T = c(3.92, 8.91),
#'                  Mean_CPM_C = c(0.04, 0.03),
#'                  log2FoldChange = c(7.22, 5.81),
#'                  lfcSE = c(0.80, 0.48),
#'                  stat = c(12.68, 17.69),
#'                  pvalue = c(7.30135e-37, 4.37011e-70),
#'                  padj = c(1.49936e-35, 1.12976e-67),
#'                  row.names = c("DLK1", "EPCAM"))
#' # input list
#' ind_deg <- list(DEG1_df = DGE1, DEG2_df = DGE2)
#' # perform meta-analysis
#' comb_pval_df <- metaRNAseq(ind_deg, test_statistic = "invnorm", BHth = 0.05, nrep = c(2,2))}
#' @family meta-analysis functions
#' @seealso \code{\link{DGE}} for DGE analysis,
#' and \url{https://cran.r-project.org/web/packages/metaRNASeq/vignettes/metaRNASeq.pdf}
#' for metaRNASeq package info.
#' @importFrom metaRNASeq fishercomb invnorm
#' @importFrom gridExtra marrangeGrob
#' @importFrom grDevices pdf dev.off
#' @importFrom graphics hist
#' @importFrom ggplot2 ggsave
#' @export


metaRNAseq <- function(ind_deg,
                       test_statistic = "fishercomb",
                       BHth = 0.05,
                       adjpval.t = 0.05,
                       nrep = NULL,
                       plot = FALSE) {

  # Check if ind_deg is a list of at least two data.frame
  if (!is.list(ind_deg)) {
    stop("ind_deg is not a list. Please provide a list of at least two data.frames")
  }
  if (length(ind_deg) < 2) {
    stop("ind_deg contains ", length(ind_deg), " data.frame. Please provide a list of at least two data.frames")
  }

  common_genes <- Reduce(intersect, lapply(ind_deg, rownames))
  if (length(common_genes) == 0) {
    stop(" your DGE data.frames do not have any shared gene")
  }

  # if test_statistic is invnorm check that nrep has the same number of elements than ind_deg
  if (test_statistic == "invnorm" && length(ind_deg) != length(nrep)) {
    stop("nrep must have the same number of elements of ind_deg.")
  }

  histp  <- list()
  rawpval <- list()

  for (i in seq_along(ind_deg)){
    if (!("pvalue" %in% colnames(ind_deg[[i]]))) {
      stop("The DGE dataframe ", names(ind_deg)[i], " must include a column named pvalue.")

    }
    ind_deg[[i]] <- ind_deg[[i]][common_genes, ]
    rawpval[[i]] <- ind_deg[[i]][["pvalue"]]
    if (plot) {
      pdf(file = paste(names(ind_deg[i]), "_raw_pval_hist.pdf", sep = "", collapse = NULL))
      hist(rawpval[[i]], breaks = 100, col = "grey", main = names(ind_deg[i]), xlab = "Raw p-values")
      dev.off()
    }
  }

  if (plot) {
    ggsave(filename = "raw_pval.pdf",
           plot = marrangeGrob(histp, nrow = 1, ncol = 1),
           device = "pdf")
  }

  if (test_statistic == "fishercomb") {
    fish_comb <- fishercomb(rawpval, BHth = BHth)
    fish_comb$DEname <- common_genes
    fish_comb$binaryadjpval <- ifelse(fish_comb$adjpval <= adjpval.t, 1, 0)
    if (plot) {
      pdf(file = paste("fishercomb_pval_hist.pdf", sep = "", collapse = NULL))
      hist(fish_comb$rawpval, breaks = 100, col = "grey",
           main = names(ind_deg), xlab = "Raw p-values")
      dev.off()
    }
    return(fish_comb)
  } else if (test_statistic == "invnorm") {
    inv_norm <- invnorm(rawpval, nrep = nrep, BHth = BHth)
    inv_norm$DEname <- common_genes
    inv_norm$binaryadjpval <- ifelse(inv_norm$adjpval <= adjpval.t, 1, 0)
    if (plot) {
      pdf(file = paste("invnorm_pval_hist.pdf", sep = "", collapse = NULL))
      hist(inv_norm$rawpval, breaks = 100, col = "grey", main = names(ind_deg),
           xlab = "Raw p-values")
      dev.off()
    }
    return(inv_norm)
  } else {
    stop("The defined test_statistic could only be either fishercomb or invnorm.")
  }

}

data(ind_deg)
data(countData)
data(metadata)
data(enrichedList)

#commented to reduce check time
#test_that("Gene2SProtein -package core function- tests", {
  # 1: Check the output for a specific SP gene
  #result1 <- Gene2SProtein(c("EPCAM"), output_tsv = FALSE)
  #expect_equal(result1$Surfaceome.Label, "surface")

  # 2: Check the output for an unknown gene, not SP coding
  #expect_warning(Gene2SProtein(c("InventedGene"), output_tsv = FALSE))
#})


test_that("Enrichment_barplot", {
  dbs <- c("GO_Cellular_Component_2021")
  expect_no_error(Enrichment_barplot(enrichedList,
                                     enrich.databases = dbs,
                                     p_adj = 0.1, num_term = 2, cond = "UP"))
})

test_that("metaRNAseq", {
  # 1: if test_statistic is invnorm check that ind_deg and nrep and have the
  # same number of elements
  expect_no_error(metaRNAseq(ind_deg,
                             test_statistic = "invnorm",
                             BHth = 0.05,
                             nrep = c(2, 2)))

  # 2: if test_statistic is invnorm nrep must be specified
  expect_error(metaRNAseq(ind_deg,
                          test_statistic = "invnorm",
                          BHth = 0.05))

  # 3: Check that input ind_deg is a list of at least two data.frames.
  # stop when not.
  expect_error(metaRNAseq(ind_deg$DEG2_df,
                          test_statistic = "fishercomb",
                          BHth = 0.05))

  # 4: Check that input ind_deg is a list of at least two data.frames.
  # continue when it is.
  expect_no_error(metaRNAseq(ind_deg,
                             test_statistic = "fishercomb",
                             BHth = 0.05))
})

#commented to reduce check time
#test_that("Annotate_SPID", {
#  websiteLive <- getOption("enrichR.live", default = FALSE)
#  # 1: Check for connectivity
#  if (websiteLive) {
#    expect_no_error(Annotate_SPID(ind_deg$DEG2_df, "WikiPathway_2023_Human"))
#  } else {
#    expect_error(Annotate_SPID(ind_deg$DEG2_df, "WikiPathway_2023_Human"))
#  }
#})

#commented to reduce check time
#test_that("combine_fisher_invnorm", {
#  invnorm <- metaRNAseq(ind_deg, test_statistic = "invnorm", BHth = 0.05, nrep = c(2,2))
#  fishercomb <- metaRNAseq(ind_deg, test_statistic = "fishercomb", BHth = 0.05)
#  # 1: Check the output on toy data
#  expect_no_error(combine_fisher_invnorm(ind_deg,
#                                         invnorm, fishercomb,
#                                         adjpval = 0.05,
#                                         output_tsv = FALSE))
#
#})

#commented to reduce check time
#test_that("DownloadArchS4", {
#  # 1: Check that downloaded counts matrix matches the query
#  GSM <- "GSM3447008"
#  GEO_count_matrix <- DownloadArchS4(GSM, species = "human",
#                                     print_tsv = FALSE, filename = NULL)
#  expect_equal(colnames(GEO_count_matrix), "GSM3447008")
#})

#commented to reduce check time
#test_that("GEOmetadata", {
#  # 1: Check that downloaded metadata matches the query
#  GSE <- "GSE121810"
#  meta <- GEOmetadata(GSE)
#  expect_equal(unique(meta$GSE), "GSE121810")
#})

test_that("SVenn", {
  # 1: Expect no error when the number of sets is up to 7 sets
  S_list <- list(SP1 <- c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1","TOP2A"),
                 SP2 <- c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"))
  expect_no_error(SVenn(S_list,
                        output_intersectionFile = FALSE,
                        cols.use = c("pink", "yellow")))

  # 2: Expect error when the number of sets is higher than 7
  S_list <- list(SP1 <- c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1","TOP2A"),
                 SP2 <- c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"),
                 SP3 <- c("MKI67","CD4","CD8"),
                 SP4 <- c("MKI67","CD4","CD8"),
                 SP5 <- c("SELP"),
                 SP6 <- c("SELE"),
                 SP7 <- c("CECAM1"),
                 SP8 <- c("EPCAM", "CD24"))
  expect_error(SVenn(S_list, output_intersectionFile = FALSE))
})

#commented to reduce check time
#test_that("Splot", {
  # 1: Check the output for specific surface protein genes
  #SurfaceProteins_df <- Gene2SProtein(ind_deg$DEG2_df$GeneID, input_type = "gene_name")
  #expect_no_error(Splot(SurfaceProteins_df))
#})

#commented to reduce check time
#test_that("TCGA_download", {
#GBM_list_s1 <- TCGA_download(project="TCGA-GBM",
#                             whichcounts = "unstranded",
#                             save.matrix = FALSE, save.metadata = FALSE,
#                             barcodes = c("TCGA-06-0878-01A-01R-1849-01"))
#expect_equal(colnames(GBM_list_s1[[1]]), "TCGA-06-0878-01A-01R-1849-01")
#})

#commented to reduce check time
#test_that("Enrichment", {
#  websiteLive <- getOption("enrichR.live", default = FALSE)
#  # 1: Check for connectivity
#  if (!websiteLive) {
#    message("EnrichR website is not reachable. Skipping test.")
#  } else {
#    result <- tryCatch({
#      Enrichment(ind_deg,
#                 enrich.databases = c("GO_Cellular_Component_2021"),
#                 save.results = FALSE)
#    }, error = function(e) {
#      message(paste("Error in Enrichment:", e$message))
#      return(NULL)
#    })
#    # 2: Check successful execution)
#    expect_true(!is.null(result))
#  }
#})

#commented to reduce check time
#test_that("plotPCA", {
#  expect_no_error(SurfR::plotPCA(matrix = cpm(countData),
#          metadata = metadata,
#          nTOP = 100,
#          dims = c(1,2),
#          color.by = "condition", shape.by = "therapy",
#          label = FALSE, main = "PCA"))
#})

#commented to reduce check time
#test_that("DGE", {
#  expect_no_error(DGE(expression = countData, metadata = metadata,
#                  Nreplica = 2,
#                  design = "~condition",condition = "condition",
#                  TEST = "A", CTRL = "B"))
#})

#' plotPCA function
#'
#' Plot PCA highlighting one or two data features
#'
#' @param matrix Filtered count matrix in CPM or RPKM with gene on the row and sample ID on the column.
#' @param metadata Sample metadata, row.names must be samples names.
#' @param nTOP number of top genes to use for principal components,
#' selected by highest row variance
#' @param dims Dimensions to plot, must be a two-length numeric vector specifying x- and y-dimensions
#' @param centering Logical. If \code{TRUE} center PCs
#' @param scaling Logical. If \code{TRUE} scales PCs
#' @param color.by Name of one or more metadata columns to color point by.
#' @param shape.by Name of one or more metadata columns to shape point by. If NULL, all points are circles \(default\).
#' @param pt.size Size of the points in the plot.
#' @param cols.use Vector of colors, each color corresponds to an identity class.
#' By default, ggplot assigns colors.
#' @param shape.use Vector of shape, each shape corresponds to an identity class.
#' @param main Plot title. Default = PCA.
#' @param label Logical. If \code{TRUE} adds samples label. Default = FALSE.
#' @param new.label If NULL, use the sample names as in metadata row.names.
#' Otherwise you can specify new labels.
#' @return PCA plot objec created by ggplot2,
#' which can be assigned and further customized.
#' @examples
#' \dontrun{
#' # Simulation of bulk RNA data
#' countData <- matrix(floor(runif(10000, min=0, max=101)),ncol=4)
#' colnames(countData) <- paste("sample", seq_len(ncol(countData)), sep = "")
#' rownames(countData) <- paste("gene", seq_along(seq_len(10000/4)), sep = "")
#' metadata <- data.frame(samplesID = paste("sample", seq_len(ncol(countData)), sep = ""),
#'                      condition = factor(c("A","A","B","B")),
#'                      therapy = factor(c("T1","T2","T1","T2")))
#' row.names(metadata) <- metadata$samplesID
#' library(edgeR)
#' SurfR::plotPCA(matrix = cpm(countData),
#'         metadata = metadata,
#'         nTOP = 100,
#'         dims = c(1,2),
#'         color.by = "condition", shape.by = "therapy",
#'         label = FALSE, main = "PCA")}
#' @family plot functions
#' @importFrom ggplot2 ggplot aes geom_point unit xlab ylab coord_fixed ggtitle theme theme_bw geom_hline geom_vline element_text element_rect scale_color_manual scale_shape_manual
#' @importFrom ggrepel geom_label_repel
#' @importFrom stats prcomp
#' @importFrom scales hue_pal
#' @export


plotPCA <- function(matrix,
                    metadata,
                    nTOP = 500,
                    dims = c(1, 2),
                    centering = TRUE,
                    scaling = TRUE,
                    color.by = NULL,
                    shape.by = NULL,
                    pt.size = 6,
                    cols.use = NULL,
                    shape.use = NULL,
                    main = "PCA",
                    label = FALSE,
                    new.label = NULL) {

  if (length(x = dims) != 2) {
    stop("'dims' must be a two-length vector")
  }

  matrix <- matrix[, match(row.names(metadata), colnames(matrix))]
  if (length(x = row.names(metadata)) != length(x = colnames(matrix))) {
    if (!all(row.names(metadata) == colnames(matrix))) {
      stop("matrix column and metadata rows don't match")
    }
  }

  # sort by variance and select topN
  vary <- apply(matrix, 1, var)
  vary_s <- sort(vary, decreasing = TRUE)
  TOP_N <- names(vary_s[seq_len(nTOP)])
  mtx_TOP <- matrix[TOP_N, ]

  # pca with prcomp
  pcx <- dims[1]
  pcy <- dims[2]
  centering <- centering
  scaling <- scaling

  # PCA
  pca <- prcomp(t(mtx_TOP), center = centering, scale = scaling)
  var <- round(matrix(((pca$sdev^2) / (sum(pca$sdev^2))), ncol = 1) * 100, 1)
  score <- as.data.frame(pca$x)

  # plot paramters
  xlab <- paste("PC", pcx, " (", var[pcx], "%)", sep = "")
  ylab <- paste("PC", pcy, " (", var[pcy], "%)", sep = "")
  names <- rownames(pca$x)

  if (is.null(x = color.by)) {
    color.by <- colnames(metadata)[1]
    warning("color.by option is NULL, colorig by first column of metadata")
  }

  score$color <- factor(metadata[, color.by])

  if (is.null(x = shape.by)) {
    score$shape <- ""
    shape.by <- "shape"
    warning("shape.by option is NULL. All points will have same shape")
  } else {
    score$shape <- factor(metadata[, shape.by])
  }

  if (is.null(x = cols.use)) {
    cols.use <- hue_pal()(length(x = levels(x = score$color)))
  } else if (length(cols.use) < length(x = levels(x = score$color))) {
    stop("you have ", length(x = levels(x = score$color)),
         " factors and supplied only ", length(cols.use), "color")
  }

  if (is.null(x = shape.use)) {
    shape.use <- c(16:25, seq_len(15))
  } else if (length(shape.use) < length(x = levels(x = score$shape))) {
    stop("you have ", length(x = levels(x = score$shape)),
         " factors and supplied only ", length(shape.use), "shape")
  }

  if (label) {
    if (is.null(x = new.label)) {
      score$sampleNames <- row.names(metadata)
    } else {
      score$sampleNames <- new.label
    }

    pca <- ggplot(score, aes(score[, pcx], y = score[, pcy], color = color,
                             shape = shape, label = sampleNames)) +
      geom_label_repel(data = score, aes(x = score[, pcx], y = score[, pcy],
                                         color = color, label = sampleNames),
                       size = 6,  box.padding = ggplot2::unit(1, "lines"),
                       point.padding = ggplot2::unit(0.1, "lines"),
                       segment.color = "grey50") +
      geom_point(size = pt.size) +
      xlab(xlab) +
      ylab(ylab) +
      coord_fixed() + ggtitle(main) +
      theme(plot.title = element_text(hjust = 0.5)) +
      theme_bw(base_size = 16) +
      geom_hline(yintercept = 0, linetype = "dashed", color = "darkgrey") +
      geom_vline(xintercept = 0, linetype = "dashed", color = "darkgrey") +
      theme(plot.title = element_text(color = "black", size = 16,
                                      face = "bold.italic"),
            axis.text.x = element_text(angle = 0, face = "bold",
                                       color = "black", size = 12, hjust = .5),
            axis.title.x = element_text(face = "bold", color = "black",
                                        size = 14),
            axis.text.y = element_text(angle = 0, face = "bold",
                                       color = "black", size = 12),
            axis.title.y = element_text(face = "bold",
                                        color = "black", size = 14),
            legend.text = element_text(face = "bold",
                                       color = "black", size = 10),
            legend.position = "right",
            panel.background = element_rect(fill = "white", colour = "black",
                                            linewidth = 1, linetype = "solid")) +
      scale_color_manual(values = cols.use, name = color.by) +
      scale_shape_manual(values = shape.use, name = shape.by)
  } else {
    pca <- ggplot(score, aes(score[, pcx], y = score[, pcy],
                             color = color, shape = shape)) +
      geom_point(size = pt.size) +
      xlab(xlab) +
      ylab(ylab) +
      coord_fixed() + ggtitle(main) +
      theme(plot.title = element_text(hjust = 0.5)) +
      theme_bw(base_size = 16) +
      geom_hline(yintercept = 0, linetype = "dashed", color = "darkgrey") +
      geom_vline(xintercept = 0, linetype = "dashed", color = "darkgrey") +
      theme(plot.title = element_text(color = "black",
                                      size = 16, face = "bold.italic"),
            axis.text.x = element_text(angle = 0, face = "bold",
                                       color = "black", size = 12, hjust = .5),
            axis.title.x = element_text(face = "bold", color = "black",
                                        size = 14),
            axis.text.y = element_text(angle = 0, face = "bold",
                                       color = "black", size = 12),
            axis.title.y = element_text(face = "bold",
                                        color = "black", size = 14),
            legend.text = element_text(face = "bold",
                                       color = "black", size = 10),
            legend.position = "right",
            panel.background = element_rect(fill = "white", colour = "black",
                                            linewidth = 1, linetype = "solid")) +
      scale_color_manual(values = cols.use, name = color.by) +
      scale_shape_manual(values = shape.use, name = shape.by)

  }

  geom_label_repel <- prcomp <- hue_pal <- aes <- color <- shape <- sampleNames <- NULL

  return(pca)
}

#' Format a string using placeholders - function from hypeR
#'
#' @param string A an unformatted string with placeholders
#' @param ... Variables to format placeholders with
#' @return A formatted string
#'
#' @examples
#' \dontrun{
#' format_str("Format with {1} and {2}", "x", "y")
#' }
#'
#' @keywords internal


.format_str <- function(string, ...) {
  args <- list(...)
  for (i in seq_along(args)) {
    pattern <- paste("\\{", i, "}", sep = "")
    replacement <- args[[i]]
    string <- gsub(pattern, replacement, string)
  }
  return(string)
}


#' Get url base for species-specific enrichr libraries  - function from hypeR
#'
#' @param db A species
#' @return A url
#'
#' @keywords internal
enrichr_urls <- function(db = c("Enrichr")) {
  switch(match.arg(db),
         "Enrichr"      = "http://maayanlab.cloud/Enrichr/{1}"
  )
}


#' Connect to the enrichr web application  - function from hypeR
#'
#' @param endpoint The url endpoint to connect to
#' @param db A species
#' @return A web response
#'
#' @importFrom httr GET http_status
#'
#' @keywords internal


enrichr_connect <- function(endpoint, db = c("Enrichr")) {
  url <- enrichr_urls(db)
  response <- httr::GET(.format_str(url, endpoint))
  if (!http_status(response)$category == "Success") {
    stop(http_status(response)$message)
  }
  return(response)
}


#' Download data from enrichr in the form of a named list - function from hypeR
#'
#' @param genesets A name corresponding to available genesets
#' @param db A species
#' @return A list of genesets
#'
#' @examples
#' ATLAS <- enrichr_download("Human_Gene_Atlas")
#'
#' @importFrom httr content
#'
#' @export


enrichr_download <- function(genesets, db=c("Enrichr")) {
  response <- enrichr_connect(.format_str("geneSetLibrary?mode=text&libraryName={1}", genesets), db)
  data <- content(response, "text")
  split <- strsplit(data, split="\n")[[1]]
  genesets <- sapply(split, function(x) strsplit(x, "\t")[[1]])
  names(genesets) <- unlist(lapply(genesets, function(x) x[1]))
  lapply(genesets, function(x) {
    genes <- x[seq_along(x)[-c(1, 2, 3)]]
    genes <- genes[genes != ""]
    unique(genes)
  })
}


#' Annotate_SPID
#'
#' Annotate Surface Protein Coding genes according to EnrichR libraries
#'
#' @param DGE Data.frame containing annotated DEG list, as the output of DGE or Gene2SProtein functions.
#' @param enrich.database String containing the EnrichR databases you would like to consult. Default: WikiPathway_2021_Human.
#' @param output_tsv Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.
#' @return A dataframe with surface protein coding \code{DEGs} annotation.
#' @examples
#' \dontrun{
#' # Deseq2 output sample
#' DGE = data.frame(GeneID = c("DLK1", "TOP2A"),
#'                  Mean_CPM_T = c(5.92, 9.91),
#'                  Mean_CPM_C = c(0.04, 0.03),
#'                  log2FoldChange = c(10.22, 8.42),
#'                  lfcSE = c(0.80, 0.48),
#'                  stat = c(12.68, 17.69),
#'                  pvalue = c(7.30135e-37, 4.37011e-70),
#'                  padj = c(1.49936e-35, 1.12976e-67))
#' annotated_DGE = Annotate_SPID(DGE, "WikiPathway_2021_Human")
#'
#' # Output of Gene2SProtein function
#' GeneNames = c("CIITA", "EPCAM", "DLK1", "CD24")
#' SurfaceProteins_df = Gene2SProtein(GeneNames, input_type = "gene_name")
#' annotated_SP = Annotate_SPID(SurfaceProteins_df, "GO_Biological_Process_2021")}
#' @section Warning:
#' Be sure that enrich.database exists.
#' @family functional-annotation functions
#' @seealso \code{\link{DGE}} function for DGE,
#' and \code{\link{Gene2SProtein}} function for Gene2SProtein analysis
#' @importFrom assertr col_concat
#' @importFrom tidyr separate_rows
#' @importFrom magrittr %>%
#' @importFrom utils write.table
#' @importFrom dplyr group_by summarise ungroup
#' @export


Annotate_SPID <- function(DGE,
                          enrich.database  = "WikiPathway_2021_Human",
                          output_tsv = FALSE) {


  websiteLive <- getOption("enrichR.live", default = FALSE)

  if (websiteLive) {
    #setEnrichrSite("Enrichr") # Human genes
    db <- listEnrichrDbs()
  } else {
    stop("enrichR website can not be reached at the moment. Please,
          check your internet connection and retry later.")
  }

  if (is.null(dim(DGE))) {
    stop("DGE is empty")
  }

  if (length(enrich.database) != 1) {
    stop("You can select only one enrich.database at a time.")
  }

  if (!(enrich.database %in% db$libraryName)) {
    stop(enrich.database, " is not a valid enrichR geneset.")
  }

  annotation_table <- enrichr_download(enrich.database)

  # here gives a warning, but we can safely ignore it.
  # the warning says we are trying to combine dataframes with a different
  # number of columns, we  know it. It is expected.
  suppressWarnings({
  annotation_table <- as.data.frame(do.call(rbind, annotation_table))})
  annotation_table["test"] <- col_concat(annotation_table, sep = " ")
  annotation_table["GeneID"] <- trimws(annotation_table$test, which = c("both"))
  annotation_table$term <- row.names(annotation_table)
  annotation_table_sub <- annotation_table[, c("term", "GeneID")]

  exploded <- unique(separate_rows(annotation_table_sub,
                                   GeneID, sep = " ", convert = FALSE))
  grouped  <-
    exploded %>%
    group_by(GeneID) %>%
    summarise(temp = toString(term)) %>%
    ungroup()

  colnames(grouped) <- c("GeneID", enrich.database)

  merged <- merge(DGE, grouped, by = "GeneID", all.x = TRUE)

  if (output_tsv) {
    write.table(merged, paste(enrich.database, "_SP_annotation.tsv", sep = "_"),
                quote = FALSE, sep = "\t")
  }
  GeneID <- term <- NULL
  return(merged)
}

#' Enrichment_barplot
#'
#' Barplot representing the top up-regulated or down-regulated significant pathways
#'
#' @param Enrich A list of enrichment tables for up and down-regulated genes in the different enrichR databases.
#' Output of Enrichment.R function for one DGE experiment.
#' @param enrich.databases Vector of EnrichR databases to consider. These databases must be present in the Enrich list.
#' @param p_adj Double. Minimum Adjusted pvalue threshold for the enrichment
#' @param num_term Double. Number of up-regulated and dw-regulated terms to represent
#' @param cond String. Title of the plot.
#' @param plot Logical. If TRUE save plot as pdf.
#' @return bar plot of significant pathways.
#' @examples
#' \dontrun{
#' dbs <- c("GO_Biological_Process_2021")
#' dfList <- list()
#' dfList[["fdr_up"]]$GO_Biological_Process_2021 <- data.frame(
#'   Term = c("peripheral nervous system neuron differentiation (GO:0048934)",
#'            "apoptotic chromosome condensation (GO:0030263)",
#'            "negative regulation of CD4-positive, alpha-beta T cell differentiation (GO:0043371)"),
#'   Overlap = c("1/5", "1/5", "1/5"),
#'   P.value = c(0.0007498315, 0.0007498315, 0.0007498315),
#'   Adjusted.P.value = c(0.00893491, 0.00893491, 0.00893491),
#'   Old.P.value = c(0, 0, 0),
#'   Old.Adjusted.P.value = c(0, 0, 0),
#'   Odds.Ratio = c(2499.125, 2499.125, 2499.125),
#'   Combined.Score = c(17982.86, 17982.86, 17982.86),
#'   Genes = c("RUNX1", "TOP2A", "RUNX1"))
#' dfList[["fdr_down"]]$GO_Biological_Process_2021 <- data.frame(
#'   Term = c("skin morphogenesis (GO:0043589)",
#'            "skin development (GO:0043588)",
#'            "collagen fibril organization (GO:0030199)"),
#'   Overlap = c("2/7", "2/80", "2/89"),
#'   P.value = c(3.149296e-07, 4.727687e-05, 5.856991e-05),
#'   Adjusted.P.value = c(1.291211e-05, 8.004554e-04, 8.004554e-04),
#'   Old.P.value = c(0, 0, 0),
#'   Old.Adjusted.P.value = c(0, 0, 0),
#'   Odds.Ratio = c(7996.8000, 510.7436, 457.7011),
#'   Combined.Score = c(119719.427, 5086.745, 4460.430),
#'   Genes = c("COL1A1;COL1A2", "COL1A1;COL1A2", "COL1A1;COL1A2"))
#' Enrichment_barplot(dfList,
#'                    enrich.databases = dbs,
#'                    p_adj = 0.01, num_term = 3, cond = "UP")
#'}
#' @family functional-annotation functions
#' @family plot functions
#' @import knitr
#' @importFrom stringr str_replace
#' @importFrom ggplot2 ggplot geom_bar aes scale_fill_gradient scale_x_discrete labs coord_flip
#' @importFrom utils head
#' @importFrom grDevices pdf dev.off
#' @export


Enrichment_barplot <- function(Enrich,
                               enrich.databases  = c("GO_Biological_Process_2021",
                                                     "GO_Cellular_Component_2021",
                                                     "GO_Molecular_Function_2021"),
                               p_adj = 0.05, num_term = 10, cond = "UP",
                               plot = FALSE) {

  enrichR.table <- data.frame()

  # check if Enrich contains enrich.databases

  if (cond == "UP") {
    enrich_list <- Enrich[["fdr_up"]]

  } else if (cond == "DOWN") {
    enrich_list <- Enrich[["fdr_down"]]
  }

  if (length(setdiff(enrich.databases, names(enrich_list))) > 0) {
    warning(setdiff(enrich.databases, names(enrich_list)), " not present in your Enrichment analysis.")
    enrich.databases <- intersect(enrich.databases, names(enrich_list))
  }

  if (length(enrich.databases) == 0) {
    stop("Selected Enrich databases not present in your enrichment analysis.")
  }

  for (dat in enrich.databases) {
    Table <- enrich_list[[dat]]
    enrichR.table <- rbind(enrichR.table, Table)
  }

  row.names(enrichR.table) <- enrichR.table$Term

  fx <- function(x) eval(parse(text = enrichR.table[x, ]$Overlap))

  p <- enrichR.table[enrichR.table$Adjusted.P.value < p_adj, "Term"]

  if (length(p) > 0) { # Discarding the not-significant results (to avoid errors)
    pathways.dataframe <- data.frame(
      Pathway = p,
      gene.ratio = vapply(p, fx, FUN.VALUE = numeric(1)),
      p.value = enrichR.table[p, ]$P.value,
      p.value.adj = enrichR.table[p, ]$Adjusted.P.value
    )
    # Formatting the dataframe for the plot
    pathways.dataframe <- pathways.dataframe[order(pathways.dataframe$p.value.adj), ]
  }

  # N Top significant pathways

  top_sig <- head(pathways.dataframe[seq_len(num_term), ], num_term)

  # Significant pathway barplot

  top_sig$Log10Adj.P.value <- -log10(top_sig$p.value.adj)

  top_sig$Pathway <- stringr::str_replace(top_sig$Pathway, "\\s*\\([^\\)]+\\)", " ")

  top_sig <- top_sig[order(top_sig$gene.ratio), ]

  p_top_sig <- ggplot(top_sig, ggplot2::aes(x = Pathway, y = gene.ratio, fill = Log10Adj.P.value)) +
    geom_bar(stat = "identity") +
    scale_fill_gradient(low = "blue", high = "red",  name = "-Log10(p.adj.value)") +
    scale_x_discrete(limits = top_sig$Pathway) +
    labs(title = cond) +
    coord_flip()

  if (plot) {
    pdf("Top_sig_Pathways_barplot.pdf", 10, 5)
    p_top_sig
    dev.off()
  }

  Pathway <- gene.ratio <- Log10Adj.P.value <- NULL

  return(p_top_sig)

}

##' Set Enrichr Website
##'
##' Set Enrichr Website
##' @title Set Enrichr Website
##' @param site site requested
##' @return Changes Enrichr Website connection
##' @author Alexander Blume
##' @export
setEnrichrSite <- function(site) {
  enrichR.base.address <- "https://maayanlab.cloud/"
  site <- gsub(enrichR.base.address, "", site)
  matched <- grep(paste0("^",site),
                  getOption("enrichR.sites"),
                  ignore.case = TRUE, value = FALSE)
  if( length(matched) == 0 ) {
    message("Given website does not match available sites: ", site)
    message("Choose from:\n",
            paste("-",getOption("enrichR.sites"), "\n"))
  } else if (length(matched) > 1) {
    message("Given website matches multiple options: ", site)
    message(paste("-", getOption("enrichR.sites")[matched], "\n"),)
  } else {
    site <- getOption("enrichR.sites")[matched]

    options(enrichR.base.address = paste0(enrichR.base.address,site,"/"))
    message("Connection changed to ",paste0(enrichR.base.address,site,"/"))
    getEnrichr(url = paste0(enrichR.base.address,"datasetStatistics"))
  }
}


##' Helper function
##'
##' Helper function for HTTP methods GET
##' @title Check if EnrichR is online
##' @param url (Required). URL address to check
##' @param ... (Optional). Additional parameters to pass to GET
##' @return Invisible response object from GET request if successful, or NULL on failure
##' @importFrom httr GET
##' @importFrom httr status_code
##' @importFrom httr http_status
getEnrichr <- function(url, ...) {
  options(enrichR.live = FALSE) # Default to offline
  tryCatch({
    # Make the GET request
    response <- GET(url = url, ...)
    # Check the status code
    if (status_code(response) == 200) {
      options(enrichR.live = TRUE) # Set to live if the request succeeds
      invisible(response)
    } else {
      message(http_status(response)$message)
      NULL
    }
  },
  warning = function(w) {
    message("Warning: ", w$message)
    NULL
  },
  error = function(e) {
    message("Error: ", e$message)
    NULL
  })
}


##' Gene enrichment using Enrichr
##'
##' Gene enrichment using Enrichr, slighthly modified by Aurora Maurizio.
##' @title Gene enrichment using Enrichr
##' @param genes (Required). Character vector of Entrez gene symbols as input. A data.frame
##' of gene symbols in first column is also acceptable, optionally a score denoting the
##' degree of membership between 0 and 1 in the second column.
##' @param databases (Required). Character vector of databases to search.
##' See https://maayanlab.cloud/Enrichr/ for available databases.
##' @return Returns a list of data.frame of enrichment terms, p-values, ...
##' @author Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
##' @importFrom httr POST
##' @importFrom httr use_proxy
##' @importFrom rjson fromJSON
##' @importFrom utils read.table
##' @export
##' @examples
##' \dontrun{
##' GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5")
##' dbs <- c("GO_Molecular_Function_2023", "GO_Cellular_Component_2023",
##'          "GO_Biological_Process_2023")
##' enriched1 <- enrichr(GeneID, dbs)
##' print(head(enriched1[[1]]))}


enrichr <- function(genes, databases) {
  if (length(genes) < 1) {
    stop("No genes have been given")
  }

  base.address <- "https://maayanlab.cloud/Enrichr/"
  enrichR.base.address <- "https://maayanlab.cloud/Enrichr/"
  getEnrichr(url = base.address)

  if (!getOption("enrichR.live")) {
    stop("Enrichr website is unreachable")
  }

  if (is.null(databases)) {
    stop("No databases have been provided")
  }

  if (is.vector(genes) & !all(genes == "") & length(genes) != 0) {
    temp <- POST(url=paste0(enrichR.base.address, "enrich"),
                 body=list(list=paste(genes, collapse="\n")))
  } else if (is.data.frame(genes)) {
    temp <- POST(url=paste0(enrichR.base.address, "enrich"),
                 body=list(list=paste(paste(genes[,1], genes[,2], sep=","),
                                      collapse="\n")))
  } else {
    warning("genes must be a non-empty vector of gene names or a data.frame with genes and score.")
  }

  dbs <- as.list(databases)
  result <- lapply(dbs, function(x) {
    r <- getEnrichr(url = paste0(base.address, "export"), query = list(file = "API", backgroundType = x))
    if (!getOption("enrichR.live")) stop("Enrichr website is unreachable")
    r <- gsub("&#39;", "'", intToUtf8(r$content))
    tc <- textConnection(r)
    r <- read.table(tc, sep = "\t", header = TRUE, quote = "", comment.char = "")
    close(tc)
    return(r)
  })

  names(result) <- databases
  return(result)
}

##' Set Enrichr Website
##'
##' Set Enrichr Website
##' @title Set Enrichr Website
##' @param site site requested
##' @return Changes Enrichr Website connection
##' @author Alexander Blume
##' @export
setEnrichrSite <- function(site) {
  enrichR.sites.base.address = "https://maayanlab.cloud/"
  site <- gsub(enrichR.sites.base.address, "", site)
  matched <- grep(paste0("^",site),
                  getOption("enrichR.sites"),
                  ignore.case = TRUE, value = FALSE)
  if( length(matched) == 0 ) {
    message("Given website does not match available sites: ", site)
    message("Choose from:\n",
            paste("-",getOption("enrichR.sites"), "\n"))
  } else if (length(matched) > 1) {
    message("Given website matches multiple options: ", site)
    message(paste("-", getOption("enrichR.sites")[matched], "\n"),)
  } else {
    site <- getOption("enrichR.sites")[matched]

    options(enrichR.base.address = paste0("https://maayanlab.cloud/Enrichr/",site,"/"))
    message("Connection changed to ",paste0("https://maayanlab.cloud/Enrichr/",site,"/"))
    getEnrichr(url = paste0(getOption("enrichR.base.address"),"datasetStatistics"))
  }
}


##' Look up available databases on Enrichr
##'
##' Look up available databases on Enrichr
##' @title Look up available databases on Enrichr
##' @return A data.frame of available Enrichr databases
##' @author Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
##' @importFrom rjson fromJSON
##' @export
##' @examples
##' dbs <- listEnrichrDbs()
listEnrichrDbs <- function() {
  dfSAF <- getOption("stringsAsFactors", FALSE)
  options(stringsAsFactors = FALSE)
  dbs <- getEnrichr(url = "https://maayanlab.cloud/Enrichr/datasetStatistics")
  if (!getOption("enrichR.live")) return()
  dbs <- intToUtf8(dbs$content)
  dbs <- fromJSON(dbs)
  dbs <- lapply(dbs$statistics, function(x) do.call(cbind.data.frame, x))
  dbs <- do.call(rbind.data.frame, dbs)
  options(stringsAsFactors = dfSAF)
  dbs
}

#' Enrichment function
#'
#' Perform enrichment Analysis of RNA-Seq Data
#'
#' @param dfList Dataframes list
#' @param enrich.databases Vector of EnrichR databases to consult
#' @param p_adj Double. Adjusted pvalue threshold for the enrichment
#' @param logFC Double. Fold change threshold for the enrichment
#' @param save.results Logical. If TRUE saves input gene lists and enrichment results.
#' @return A list of enrichment tables for upregulated and downregulated genes
#' in the different enrichr databases
#' @examples
#' \dontrun{
#' df1 <- data.frame(GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5"),
#'                   baseMean = c(13490.22, 10490.23, 8888.33, 750.33),
#'                   log2FoldChange = c(5.78, 6.76, -7.78, -8.78),
#'                   padj = c(2.28e-143, 2.18e-115, 2.18e-45, 0.006),
#'                   row.names = c("MEST", "CDK1", "PCLAF", "BIRC5"))
#' df2 <- data.frame(GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5"),
#'                   baseMean = c(13490.22, 10490.23, 8888.33, 750.33),
#'                   log2FoldChange = c(5.78, 6.76, -7.78, -8.78),
#'                   padj = c(2.28e-143, 2.18e-115, 2.18e-45, 0.006),
#'                   row.names = c("MEST", "CDK1", "PCLAF", "BIRC5"))
#' dfList <- list(df1 = df1, df2 = df2)
#' test <- Enrichment(dfList, enrich.databases = c("GO_Cellular_Component_2021"),
#'                    save.results = FALSE)}
#' @family functional-annotation functions
#' @seealso \url{https://maayanlab.cloud/Enrichr/} for additional information about enrichR.
#' @importFrom openxlsx write.xlsx
#' @importFrom utils write.table
#' @export

Enrichment <- function(dfList, enrich.databases  = c("GO_Biological_Process_2021",
                                                     "GO_Cellular_Component_2021",
                                                     "GO_Molecular_Function_2021",
                                                     "KEGG_2021_Human",
                                                     "MSigDB_Hallmark_2020",
                                                     "WikiPathways_2016",
                                                     "BioCarta_2016",
                                                     "Jensen_TISSUES",
                                                     "Jensen_COMPARTMENTS",
                                                     "Jensen_DISEASES"),
                       p_adj = 0.05, logFC = 1,
                       save.results = FALSE) {


  websiteLive <- getOption("enrichR.live", default = FALSE)

  if (websiteLive) {
    setEnrichrSite("Enrichr") # Human genes
    db <- listEnrichrDbs()
  } else {
    stop("enrichR website can not be reached at the moment. Please,
          check your internet connection and retry later.")
  }

  enrichr.list <- list()

  if (length(setdiff(enrich.databases, db$libraryName)) > 0) {
    warning(setdiff(enrich.databases, db$libraryName), " is not an enrichR geneset and will be removed.\n")
    enrich.databases <- intersect(enrich.databases, db$libraryName)
  }

  if (length(enrich.databases) == 0) {
    stop("Please provide at least one valid enrich.database.")
  }

  for (i in names(dfList)) {
    df_obj <- dfList[[i]]
    signif <- (df_obj[df_obj$padj <= p_adj, ])
    number_of_sig_genes  <- nrow(signif)

    message(i, " ", number_of_sig_genes, " significant genes\n")

    if (number_of_sig_genes == 0) {
      stop("no significant genes found. Enrichment can't be performed.")
    }

    neg <- nrow(signif[signif$log2FoldChange < logFC, ])

    message(i, " ", neg, " negative fold change\n")

    neg_list <- rownames(signif[signif$log2FoldChange < logFC, ])

    if (length(neg_list) == 0) {
      warning("There are no significantly downregulated genes in ", i)
  } else {
    if (save.results) {
      dir.create("enrichR/", showWarnings = FALSE, recursive = TRUE)
      write.table(neg_list, paste("./enrichR/FDRdown_", i,
                                  ".txt", sep = ""), quote = FALSE,
                  row.names = FALSE, col.names = FALSE)
      }

    }

    pos  <- nrow(signif[signif$log2FoldChange > logFC, ])
    message(i, " ", pos, " positive fold change\n")

    pos_list  <- rownames(signif[signif$log2FoldChange > logFC, ])

    if (length(pos_list) == 0) {
      warning("There are no significantly upregulated genes in ", i)
    } else {
      if (save.results) {
      dir.create("enrichR/", showWarnings = FALSE, recursive = TRUE)
      write.table(pos_list, paste("./enrichR/FDRup_", i,
                                  ".txt", sep = ""), quote = FALSE,
                  row.names = FALSE, col.names = FALSE)
      }

    }

    enrichr.list[[i]] <- lapply(list(pos_list, neg_list), function(x) {
      enrichr(genes = x, databases = enrich.databases)
    })
    names(enrichr.list[[i]]) <-  c("fdr_up", "fdr_down")

  }

  if (save.results) {
    dir.create("enrichR/", showWarnings = FALSE, recursive = TRUE)
    for (i in names(dfList)) {
      for (j in c("fdr_up", "fdr_down")){
        filename <- paste("./enrichR/", i, j, ".xlsx", sep = "")
        if (!is.null(enrichr.list[[i]][[j]])) {
          write.xlsx(x = enrichr.list[[i]][[j]], file = filename)
        }
      }
    }
  }

  return(enrichr.list)
}

#' DGE function
#'
#' Perform Differential Gene Expression Analysis of RNA-Seq Data
#'
#' @param expression Dataframe with counts
#' @param metadata Dataframe with sample metadata
#' @param Nreplica  Double. Minimum number of replicates in each group
#' @param design Design formula for DGE
#' @param condition Column of the metadata ti use for DGE results
#' @param TEST Character. sample name in metadata
#' @param CTRL Character. sample name in metadata
#' @param FC_filt Dataframe with counts
#' @param alpha Double. the significance cutoff used for optimizing the independent filtering (by default 0.1).
#' If the adjusted p-value cutoff (FDR) will be a value other than 0.1, alpha should be set to that value.
#' @param output_tsv Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.
#' @param output_filename Name of the tsv output file. Default is DEGs.tsv.
#' @return A dataframe with \code{DEGs}
#' @examples
#' \dontrun{
#' # Simulation of bulk RNA data
#' countData <- matrix(floor(runif(10000, min=0, max=101)),ncol=4)
#' colnames(countData) <- paste("sample", seq_len(ncol(countData)), sep = "")
#' rownames(countData) <- paste("gene", seq_along(seq_len(10000/4)), sep = "")
#' metadata <- data.frame(samplesID = paste("sample", seq_len(ncol(countData)), sep = ""),
#'                        condition = factor(c("A","A","B","B")))
#' row.names(metadata) <- metadata$samplesID
#' # Perform DGE
#' DGEresults <- DGE(expression = countData, metadata = metadata,
#'                  Nreplica = 2,
#'                  design = "~condition",condition = "condition",
#'                  TEST = "A", CTRL = "B")}
#' @importFrom DESeq2 DESeqDataSetFromMatrix DESeq counts results
#' @importFrom edgeR cpm
#' @importFrom utils write.table
#' @importFrom stats as.formula
#' @import SPsimSeq
#' @export


DGE <- function(expression,
                metadata,
                Nreplica,
                design = "~condition",
                condition = "condition",
                TEST,
                CTRL,
                alpha = 0.05,
                FC_filt = 0,
                output_tsv = FALSE,
                output_filename = "DEGs.tsv") {

  # check the match between TEST CTRL and metadata
  if (length(c(CTRL, TEST) %in% metadata[, condition]) == 0) {
    stop("In your metadata the column ", condition, " does not contain ",
         CTRL, " and ", TEST, ".")
  } else if (length(c(CTRL) %in% metadata[, condition]) == 0) {
    stop("In your metadata the column ", condition, " does not contain ",
         CTRL, ".")
  } else if (length(c(TEST) %in% metadata[, condition]) == 0) {
    stop("In your metadata the column ", condition, " does not contain ",
         TEST, ".")
  }

  if (!(setequal(row.names(metadata), colnames(expression)))) {
    stop("row.names of metadata must be equal to expression matrix colnames")
  }

  # perform DGE
  design.formula <- as.formula(design)
  dds <- DESeqDataSetFromMatrix(countData = expression,
                                colData = metadata,
                                design = design.formula)

  min.samples <- Nreplica
  keep <- rowSums(cpm(counts(dds)) >= 1) >= min.samples
  dds <- dds[keep, ]
  cpm <- cpm(counts(dds), log = FALSE)
  dga <- DESeq(object = dds,
               test = "Wald",
               fitType = "parametric",
               betaPrior = FALSE,
               minReplicatesForReplace = Inf)

  dgeResults <- results(dga,
                        contrast = c(condition, TEST, CTRL),
                        cooksCutoff          = Inf,
                        independentFiltering = TRUE,
                        alpha                = alpha,
                        pAdjustMethod        = "BH")

  dgeResults <- dgeResults[order(dgeResults$pvalue, decreasing = FALSE), ]

  dgeResults$Mean_CPM_C <- rowMeans(cpm[row.names(dgeResults),
                                        row.names(metadata[metadata[, condition] == CTRL, ])])
  dgeResults$Mean_CPM_T <- rowMeans(cpm[row.names(dgeResults),
                                        row.names(metadata[metadata[, condition] == TEST, ])])

  dgeResults$GeneID <- row.names(dgeResults)
  dgeResults <- dgeResults[order(dgeResults$log2FoldChange, decreasing = TRUE),
                           c("GeneID", "Mean_CPM_T", "Mean_CPM_C",
                             "log2FoldChange", "lfcSE", "stat",
                             "pvalue", "padj")]

  # tsv

  if (output_tsv) {
    write.table(dgeResults, output_filename, quote = FALSE, sep = "\t")
  }

  return(dgeResults)
}

#' combine_fisher_invnorm
#'
#' Combine Meta-Analysis results with individual DE tables
#'
#' @param ind_deg List of indipendent DEG dataframes with p-values to be combined.
#' @param invnorm inverse normal p-value combination technique dataframe (output of metaRNAseq)
#' @param fishercomb Fisher p-value combination technique dataframe (output of metaRNAseq)
#' @param adjpval threshold to represent as binary the Meta-Analysis output adjpval.
#' @param output_tsv logical. If TRUE, it outputs table with results. Default: TRUE
#' @param output_filename File name for the results file.
#' @return A dataframe with \code{DEindices} and \code{DEname} of DEG at the chosen Benjamini Hochberg threshold, and
#' \code{TestStatistic}, \code{rawpval}, \code{adjpval}, \code{binaryadjpval} vectors for differential expression in the meta-analysis.
#' @examples
#' \dontrun{
#' # Deseq2 output samples
#' DGE1 <- data.frame(GeneID = c("DLK1", "EPCAM"),
#'                  Mean_CPM_T = c(5.92, 9.91),
#'                  Mean_CPM_C = c(0.04, 0.03),
#'                  log2FoldChange = c(10.22, 8.42),
#'                  lfcSE = c(0.80, 0.48),
#'                  stat = c(12.68, 17.69),
#'                  pvalue = c(7.30135e-37, 4.37011e-70),
#'                  padj = c(1.49936e-35, 1.12976e-67),
#'                  row.names = c("DLK1", "EPCAM"))
#' DGE2 <- data.frame(GeneID = c("DLK1", "EPCAM"),
#'                  Mean_CPM_T = c(3.92, 8.91),
#'                  Mean_CPM_C = c(0.04, 0.03),
#'                  log2FoldChange = c(7.22, 5.81),
#'                  lfcSE = c(0.80, 0.48),
#'                  stat = c(12.68, 17.69),
#'                  pvalue = c(7.30135e-37, 4.37011e-70),
#'                  padj = c(1.49936e-35, 1.12976e-67),
#'                  row.names = c("DLK1", "EPCAM"))
#' # input list
#' ind_deg <- list(DEG1_df = DGE1, DEG2_df = DGE2)
#' # perform invnorm meta-analysis
#' invnorm <- metaRNAseq(ind_deg, test_statistic = "invnorm", BHth = 0.05, nrep = c(2,2))
#' # perform fishercomb meta-analysis
#' fishercomb <- metaRNAseq(ind_deg, test_statistic = "fishercomb", BHth = 0.05)
#' # combine results
#' comb_pval_df <- combine_fisher_invnorm(ind_deg,
#'                                       invnorm, fishercomb,
#'                                       adjpval = 0.05,
#'                                       output_tsv = FALSE)}
#' @family meta-analysis functions
#' @seealso \code{\link{DGE}} function for DGE analysis,
#' and \url{https://cran.r-project.org/web/packages/metaRNASeq/vignettes/metaRNASeq.pdf}
#' for metaRNASeq package info
#' @importFrom dplyr relocate
#' @importFrom magrittr %>%
#' @importFrom utils write.table
#' @export


combine_fisher_invnorm <- function(ind_deg,
                                   invnorm,
                                   fishercomb,
                                   adjpval = 0.05,
                                   output_tsv = TRUE,
                                   output_filename = "combine_fisher_invnorm.tsv") {

  # Check if ind_deg is a list of at least two data.frame

  if (!is.list(ind_deg)) {
    stop("ind_deg is not a list. Please provide a list of at least two data.frames.")
  }
  if (length(ind_deg) < 2) {
    stop("ind_deg contains ", length(ind_deg), " data.frame. Please provide a list of at least 2 data.frames.")
  }

  common_genes <- Reduce(intersect, lapply(ind_deg, rownames))
  # check if common_genes is empty
  if (length(common_genes) == 0) {
    stop("your DGE data.frames do not have common genes.")
  }

  DE <- data.frame(genes = common_genes)
  FC <- data.frame(genes = common_genes)

  for (i in seq_along(ind_deg)) {
    ind_deg[[i]] <- ind_deg [[i]][common_genes, ]
    ind_deg[[i]][["binarypadj"]] <- ifelse(ind_deg[[i]][["padj"]] <= adjpval, 1, 0)
    FC[[paste(names(ind_deg[i]), "log2FC", sep = "_")]] <- ind_deg[[i]][["log2FoldChange"]]
    DE[[paste(names(ind_deg[i]), "binarypadj", sep = "_")]] <- ind_deg[[i]][["binarypadj"]]
  }

  FC <- FC[-c(1)]
  DE <- DE[-c(1)]

  signsFC <- mapply(FC, FUN = function(x) sign(x))
  sumsigns <- apply(signsFC, 1, sum)
  commonsgnFC <- ifelse(abs(sumsigns) == dim(signsFC)[2], sign(sumsigns), 0)
  FC$signFC <- commonsgnFC

  comb <- cbind(FC, DE)

  comb$DE_fishercomb <- fishercomb$binaryadjpval
  comb$DE_invnorm <- invnorm$binaryadjpval
  comb$GeneID <- common_genes

  comb <- comb %>% relocate(GeneID)

  # -------- tsv --------
  if (output_tsv) {
    write.table(comb, output_filename, quote = FALSE, sep = "\t")
  }

  GeneID <- NULL

  return(comb)

}

#' SVenn
#'
#' Venn diagram of common surface proteins overexpressed among up to 7 different studies
#'
#' @param S_list A list of a maximum of 7 surface protein sets detected in different studies.
#' @param cols.use Vector of colors, each color corresponds to a study.
#' By default, ggplot assigns colors.
#' @param opacity Degree of opacity for the colors specified with cols.use (less opacity, more transparency).
#' @param output_intersectionFile logical. If TRUE (default) write an xlsx output of protein in the intersections.
#' @param filename Name of the output file with the intersections.
#' @return venn plot of common genes.
#' @examples
#' \dontrun{
#' S_list <- list(SP1 <- c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1"),
#'               SP2 <- c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"))
#' SP <- SVenn(S_list, cols.use = c("pink", "yellow"), output_intersectionFile = FALSE)}
#' @family plot functions
#' @importFrom venn venn
#' @importFrom openxlsx write.xlsx
#' @importFrom scales hue_pal
#' @seealso \code{\link{Gene2SProtein}} for detection of Surface proteins from a list of genes.
#' @export


SVenn <- function(S_list, cols.use = NULL,
                  opacity = 0.5,
                  output_intersectionFile = TRUE,
                  filename = "intersection.xlsx") {

  if (length(S_list) > 7) {
    stop("This function can plot Venn diagram with up to 7 sets")
  }

  if (is.null(x = cols.use)) {
    cols.use <- hue_pal()(length(x = names(S_list)))
  } else if (length(cols.use) < length(x = names(S_list))) {

    stop("you have ", length(x = names(S_list)),
         " unique elements and supplied only ", length(cols.use), " color \n")
  }


    venn(S_list,
         opacity = opacity,
         box = FALSE,
         ilabels = "counts",
         zcolor = cols.use,
         ilcs = 1.5,
         sncs = 1.5,
         ggplot = FALSE)

    # Table down pathway intersection
    if (output_intersectionFile) {
      list_intersection <- attr(x = venn(S_list,
                                         intersections = TRUE,
                                         opacity = opacity,
                                         box = FALSE,
                                         ilabels = "counts",
                                         zcolor = cols.use,
                                         ilcs = 1.5,
                                         sncs = 1.5,
                                         ggplot = FALSE),
                                         "intersections")

      df <- data.frame()
      for (intersection in names(list_intersection)) {
        entry <- data.frame(Intersection = intersection,
                            SurfaceProteins = list_intersection[[intersection]])
        df <- rbind(df, entry)
      }
      write.xlsx(df, filename, asTable = TRUE, overwrite = TRUE)
    }
}

#' GEOmetadata function
#'
#' Download metadata from \url{https://www.ncbi.nlm.nih.gov/geo},
#' given an input GEO accession series.
#'
#' @param GSE The GSE series ID.
#' @param GPL The GPL series numbers.
#' Required only if the chosen GSE series ID include data from multiple sequencing platforms.
#' @return A dataframe with all the available characteristics in GEO metadata \code{genes} array.
#' @examples
#' # only one sequencing platform
#' \dontrun{
#' mGSE133671 <- GEOmetadata(GSE = "GSE133671")
#' # multiple sequencing platforms
#' mGSE59483 <- GEOmetadata("GSE59483", GPL = c("GPL11154", "GPL15520"))}
#' @section Warning:
#' If the GEO accession series has more than 1 sequencing platforms you need to
#' specify the GPL series numbers.
#' @seealso
#' \url{https://www.ncbi.nlm.nih.gov/geo} for info on GEO repository
#' @family public-data functions
#' @importFrom stringr str_remove_all str_sub str_replace_all str_split
#' @importFrom utils read.table
#' @import BiocFileCache
#' @export


GEOmetadata <- function(GSE, GPL = "") {

  metafromfile <- function(file) {
    metaHEAD <- read.table(file, check.names = FALSE, sep = "\t", skipNul = TRUE, fill = TRUE)
    N_head <- grep(pattern = "Sample_title", metaHEAD$V1)
    meta <- read.table(file, skip = N_head - 1, check.names = FALSE, sep = "\t", skipNul = TRUE, fill = TRUE)
    GSMid <- grep(pattern = "Sample_geo_accession", x = meta$V1)
    title <- grep(pattern = "Sample_title", x = meta$V1)
    GPLid <- grep(pattern = "Sample_platform_id", x = meta$V1)
    c2select <- grep(pattern = "Sample_characteristics", x = meta$V1)
    row2select <- c(title, GSMid, GPLid, c2select)
    metadata_2_format <- t(meta[row2select, ])

    parentesis <- grep(pattern = "\\(", metadata_2_format["V2", ])
    metadata_2_format[, parentesis] <- str_remove_all(string = metadata_2_format[, parentesis], pattern = "\\(")
    metadata_2_format[, parentesis] <- str_remove_all(string = metadata_2_format[, parentesis], pattern = "\\)")

    metadata <- as.data.frame(metadata_2_format[2:dim(metadata_2_format)[1], ])
    metadata$GSE <- GSE
    metadata <- metadata[, c("GSE", colnames(metadata)[seq_along(colnames(metadata)) - 1])]
    cnames <- c("GSE", "title", "GSM", "GPL")

    # replace empty spaces with NA
    metadata[metadata == ""] <- NA

    for (i in seq.int(from = 5, to = length(colnames(metadata)))) {

      item <- unlist(lapply(str_split(metadata[, i], pattern = ": "), `[[`, 1))
      # check if the characteristic is the same
      if (length(unique(item[!is.na(item)])) == 1) {
        name <- unique(item[!is.na(item)])
      } else {
        name <- "multi_characteristic"

        topaste <- paste(unique(item[!is.na(item)]), collapse = ", ")
        warning(i, "th ",
                "characteristic column in ", GSE,
                " metadata presents multiple variables: ",
                topaste)

      }

      # check for NA
      metadata[, i][is.na(metadata[, i])] <- paste(name, ": NA")

      metadata[, i]  <- unlist(lapply(str_split(metadata[, i],
                                                pattern = ": "), `[[`, 2))
      cnames <- c(cnames, name)
    }

    colnames(metadata) <- cnames
    row.names(metadata) <- metadata$GSM
    colnames(metadata) <- str_replace_all(string = colnames(metadata),
                                          pattern = " ", replacement = "_")
    return(metadata)
  }

  bfc <- BiocFileCache(ask = FALSE)

  if (length(GPL) == 1 && GPL == "") {

    # only one sequencing platform
    file <- paste(GSE, "_series_matrix.txt.gz", sep = "")

    url <- paste("https://ftp.ncbi.nlm.nih.gov/geo/series/",
                            str_sub(GSE, start = 1, end = -4), "nnn/",
                            GSE, "/matrix/", file, sep = "")

    path <- bfcrpath(bfc, url)

    metadata <- metafromfile(path)

  } else {
    # multiple sequencing platforms
    metadata_i <- list()

    for (gpl in GPL) {
      file <- paste(GSE, "-", gpl, "_series_matrix.txt.gz", sep = "")

      # File not in cache, download it
      url <- paste("https://ftp.ncbi.nlm.nih.gov/geo/series/",
                              str_sub(GSE, start = 1, end = -4), "nnn/",
                              GSE, "/matrix/", file, sep = "")

      path <- bfcrpath(bfc, url)
      metadata_i[[gpl]] <- metafromfile(path)
      }
  metadata <- Reduce(f = rbind, metadata_i)
  }

  str_remove_all <- str_sub <- str_split <- str_replace_all <- read.table <- NULL
  return(metadata)
}

#' Gene2SProtein function
#'
#' Detect Surface Proteins from a vector of genes.
#' The surface proteins are identified according to the in silico human surfaceome database,
#' available at \url{https://wlab.ethz.ch/surfaceome}.
#'
#' @param genes A vector of genes.
#' @param input_type The gene identification type:
#' \code{gene_name}, \code{ensembl}, \code{entrez} or \code{uniProt_name}.
#' By default: \code{gene_name}.
#' @param output_tsv Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.
#' @param output_filename Name of the tsv output file. Default is surfaceProteins.tsv.
#' @param Surfy_version  The version of surfy dataframe you wish to use. Choose between \code{log} or \code{newest}.
#' By default use the most recent \code{log} version.
#' If a log dataframe does not exist the \code{newest} is downloaded from \url{https://wlab.ethz.ch/surfaceome}.
#' @return A data frame with filtered surface proteins from the \code{genes} array.
#' The dataframe contains also addition information obtained from surfy.
#' @examples
#'  \dontrun{
#'  # from gene name IDs to Surface proteins
#'  GeneNames <- c("CIITA", "EPCAM", "DLK1", "CD24", "CDCP1", "LYVE1", "ABCD1", "VAMP1")
#'  SurfaceProteins_df <- Gene2SProtein(GeneNames, input_type = "gene_name")
#'
#'  # from ensembl IDs to Surface proteins
#'  Ensembl <- c("ENSG00000178343", "ENSG00000176895", "ENSG00000162419", "ENSG00000170776",
#'              "ENSG00000092529", "ENSG00000135926", "ENSG00000152595", "ENSG00000121577",
#'              "ENSG00000186094", "ENSG00000126773", "ENSG00000198918", "ENSG00000167378",
#'              "ENSG00000095574", "ENSG00000140678", "ENSG00000262484", "ENSG00000133739",
#'              "ENSG00000172469", "ENSG00000112992", "ENSG00000148343", "ENSG00000138593")
#'  SurfaceProteins_df <- Gene2SProtein(Ensembl, input_type = "ensembl",
#'                                    output_tsv = FALSE, Surfy_version = "new")}
#' @section Warning:
#' The surfy database is interrogated using the gene identification type of your preference
#' between \code{gene_name}, \code{ensembl}, \code{entrez} or \code{uniProt_name}. Note that
#' you might loose some matches due to different gene version IDs.
#' @importFrom openxlsx read.xlsx write.xlsx
#' @importFrom utils write.table
#' @import BiocFileCache
#' @seealso \code{\link{DGE}} for DGE analysis,
#' \url{https://wlab.ethz.ch/surfaceome} for info on Surfy
#' @export


Gene2SProtein <- function(genes,
                          input_type = "gene_name",
                          output_tsv = FALSE,
                          output_filename = "surfaceProteins.tsv",
                          Surfy_version = "log") {

  now <- Sys.Date()
  log_list <- sort(list.files(".log/", pattern = "*.xlsx"))

  # surfy database versioning

  if (!Surfy_version %in% c("log", "new")) {
    stop("The specified database version is not available. \n  Choose between log or new")
  }

  if (length(log_list) > 0 && Surfy_version == "log") {
    #if several versions are present in .log/ directory, access the most recent
    #log file based on the sorting order of the filenames.
    ST <- read.xlsx(xlsxFile = paste0(".log/", log_list[length(log_list)]),
                    startRow = 1)
  } else {
    dir.create(".log/", recursive = TRUE, showWarnings = FALSE)
    surfaceome_table_url <- "https://wlab.ethz.ch/surfaceome/table_S3_surfaceome.xlsx"

    bfc <- BiocFileCache(ask = FALSE)

    path <- bfcrpath(bfc, surfaceome_table_url)

    ST <- read.xlsx(xlsxFile = path,
                    sheet = 1,
                    startRow = 2)

    write.xlsx(ST, paste0(".log/", "table_S3_surfaceome_", now, ".xlsx"), overwrite = TRUE)
  }

  # input identification type

  # check gene identification type
  if (!input_type %in% c("gene_name", "entrez", "ensembl", "uniProt_name")) {
    stop("The specified input type is not available. \n Choose between gene_name, entrez, ensembl or uniProt_name.")
  }
  type <- if (input_type == "gene_name") {
                                          "UniProt.gene"} else if (input_type == "entrez") {
    "GeneID"
  } else if (input_type == "ensembl") {
                                       "Ensembl.gene" } else if (input_type == "uniProt_name") {
    "UniProt.name"
  }

  # ---- filter out NA value from database ----
  # GeneID and UniProt.name

  ST <- ST[!is.na(ST$Surfaceome.Label), ]
  ST <- ST[!is.na(ST$UniProt.name), ]
  ST <- ST[!is.na(ST[, type]), ]
  row.names(ST) <- ST$UniProt.name

  # filter the database for the input genes and checks
  proteins <- ST[ST[, type] %in% genes, ]

  # check size proteins data.frame
  if (dim(proteins)[1] == 0) {

    warning("The input genes ",
            "do not have any match ",
            "in the surfaceome database. \n ",
            "Check gene alias alias and input type! ")
    surface.proteins <- data.frame(matrix(nrow = 0,
                                          ncol = length(colnames(ST))))
    colnames(surface.proteins)  <- colnames(ST)
  } else {
    # filter surface proteins and checks
    surface.proteins <- proteins[proteins$Surfaceome.Label == "surface", ]

    if (dim(surface.proteins)[1] == 0) {

      warning("No surface proteins were found among your list of genes")
    } else {

      message(dim(surface.proteins)[1], " out of ", length(genes), " genes ",
              "have a matching surface protein")
    }

  }

  # rename columns for label consistency
  surface.proteins$entrezID <- surface.proteins$GeneID

  surface.proteins$GeneID <- surface.proteins$UniProt.gene

  # tsv
  if (output_tsv) {
    write.table(surface.proteins, output_filename, quote = FALSE, sep = "\t")
  }

  read.xlsx <- write.xlsx <- write.table <- NULL
  return(surface.proteins)
}

#' TCGA_download function
#'
#' Downloads count matrix data from TCGA
#'
#' @param project, Character. A valid project from TCGAbiolinks:::getGDCprojects()$project_id
#' @param whichcounts Character. Counts data to use. Choose from: unstranded, stranded_first,stranded_second. By default, unstranded.
#' @param barcodes Character. A vector with names of the barcodes you want to download. If NULL (default) it downloads all the available barcodes in the project.
#' @param save.matrix Logical. If \code{TRUE}, outputs a tsv file with the Matrix. By default, FALSE.
#' @param save.metadata Logical. If \code{TRUE}, outputs a tsv file with the metadata. By default, FALSE.
#' @return A list containing the Matrix and the metadata.
#' @importFrom TCGAbiolinks GDCquery GDCdownload GDCprepare
#' @importFrom SummarizedExperiment assay
#' @importFrom utils write.table
#' @import biomaRt
#' @examples
#' \dontrun{
#' GBM_list_s1 <- TCGA_download(project="TCGA-GBM",
#'                              whichcounts = "unstranded",
#'                              save.matrix = FALSE, save.metadata = FALSE,
#'                              barcodes = c("TCGA-06-0878-01A-01R-1849-01"))
#' remove downloaded data from TCGA
#' unlink('GDCdata', recursive = TRUE, force = TRUE)
#' file.remove("MANIFEST.txt")
#'}
#' @family public-data functions
#' @export


TCGA_download <- function(project,
                          whichcounts = "unstranded",
                          save.matrix = FALSE,
                          save.metadata = FALSE,
                          barcodes = NULL) {

  if (is.null(barcodes)) {
    query <- GDCquery(project,
                      data.category = "Transcriptome Profiling",
                      data.type = "Gene Expression Quantification",
                      sample.type = c("Solid Tissue Normal", "Primary Tumor"),
                      workflow.type = "STAR - Counts")
  } else {
    query <- GDCquery(project,
                      data.category = "Transcriptome Profiling",
                      data.type = "Gene Expression Quantification",
                      sample.type = c("Solid Tissue Normal", "Primary Tumor"),
                      workflow.type = "STAR - Counts",
                      barcode = barcodes)
  }

  tryCatch(GDCdownload(query, method = "client"),
           error = function(e) GDCdownload(query))
  data <- GDCprepare(query, summarizedExperiment = TRUE)

  dir <- "TCGA/"

  # Save matrix
  Matrix <- assay(data, whichcounts)
  row.names(Matrix) <- data@rowRanges$gene_name

  # Save metadata
  metadata <- as.data.frame(data@colData)

  if (save.matrix) {
    dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    write.table(Matrix, file = paste0(dir, project, "_expression.tsv"),
                sep = "\t", quote = FALSE)
  }

  if (save.metadata) {
    dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    write.table(metadata, file = paste0(dir, project, "_metadata.tsv",
                                             sep = ""),
                     sep = "\t", quote = FALSE)
  }

  mat.met <- list(Matrix, metadata)

  return(mat.met)
}

#' Splot function
#'
#' Plot a barplot with features of Surface Protein
#'
#' @param SurfaceProteins_df Output dataframe of Gene2SProtein function.
#' @param group.by Name of columns to plot. Default = Membranome.Almen.main-class.
#' @param cols.use Vector of colors, each color corresponds to an identity class.
#' By default, ggplot assigns colors.
#' @param main Plot title. Default = Almen main class.
#' @return plot objec created by ggplot2,
#' which can be assigned and further customized.
#' @examples
#' \dontrun{
#' GeneNames <- c("CIITA", "EPCAM", "DLK1", "CD24", "CDCP1", "LYVE1", "ABCD1", "VAMP1")
#' SurfaceProteins_df <- Gene2SProtein(GeneNames, input_type = "gene_name")
#' Splot(SurfaceProteins_df)}
#' @family plot functions
#' @importFrom ggplot2 ggplot geom_bar aes theme element_text element_rect labs
#' @importFrom scales hue_pal
#' @export


Splot <- function(SurfaceProteins_df,
                  group.by = "Membranome.Almen.main-class",
                  cols.use = NULL,
                  main = "Almen main class") {

  SurfaceProteins_df$v2plot <- SurfaceProteins_df[, group.by]

  if (is.null(x = cols.use)) {
    cols.use <- hue_pal()(length(x = levels(factor(SurfaceProteins_df$v2plot))))
    if (length(which(is.na(SurfaceProteins_df$v2plot)) > 0)) {
      warning("NA value in your dataframe")
      # Add grey for NA protein
      cols.use <- c(cols.use, "#CCCCCC")
    }
  } else if (length(cols.use) < length(x = unique(SurfaceProteins_df$v2plot))) {

    stop("you have ", length(x = unique(SurfaceProteins_df$v2plot)),
         " unique elements and supplied only ", length(cols.use), " color \n",
         " Be carefull to NA value")
  }

  plot <- ggplot(SurfaceProteins_df, aes(x = v2plot)) +
    geom_bar(color = "black", fill = cols.use) +
    theme(plot.title = element_text(color = "black", size = 18, face = "bold.italic"),
          axis.text.x = element_text(angle = 45, face = "bold",
                                     color = "black", size = 12,
                                     vjust = 1, hjust = 1),
          axis.title.x = element_text(face = "bold", color = "black", size = 14),
          axis.text.y = element_text(angle = 0, face = "bold", color = "black",
                                     size = 12),
          axis.title.y = element_text(face = "bold", color = "black", size = 14),
          legend.text = element_text(face = "bold", color = "black", size = 10),
          legend.position = "top",
          panel.background = element_rect(fill = "white",
                                          colour = "black",
                                          linewidth = 1,
                                          linetype = "solid")) +
    labs(x = group.by, y = "N", title = main)

  hue_pal <- aes <- v2plot <- NULL

  return(plot)
}

#' DownloadArchS4 function
#'
#' Download count matrix from \url{https://maayanlab.cloud/archs4/},
#' given a vector of input GEO Sample accessions numbers (GSM).
#'
#' @param GSM Vector with the GSM ids of the samples to consider.
#' @param species Specify the specie of yuor GSM samples. Either human or mouse.
#' @param print_tsv Logical. If \code{TRUE}, outputs a tsv file with the count matrix. By default, FALSE.
#' @param filename Name of the tsv output file. Default is matrix.tsv.
#' @return A count matrix with gene on the row and GSM ID on the column.
#' @section Warning:
#' If the defined GSM ids do not have any match in ArchS4 database, we suggest to contact ArchS4 curator to add them.
#' @examples
#' \dontrun{
#' GSM <- c("GSM3447008", "GSM3447009")
#' GEO_count_matrix <- DownloadArchS4(GSM, species = "human",
#'                                   print_tsv = FALSE, filename = NULL)}
#' @seealso
#' \code{\link{GEOmetadata}} function for downloading GEO metadata.
#' \url{https://www.ncbi.nlm.nih.gov/geo} for info on GSM.
#' \url{https://maayanlab.cloud/archs4/} for info on ArchS4.
#' @family public-data functions
#' @importFrom rhdf5 h5read H5close
#' @importFrom utils write.table
#' @export


DownloadArchS4 <- function(GSM, species, print_tsv = FALSE, filename = NULL) {

  options(timeout = 600)

  matrixh5_url <- paste0("https://mssm-seq-matrix.s3.amazonaws.com/", species, "_matrix.h5")

  samples <- h5read(matrixh5_url,
                    name = "meta/Sample_geo_accession",
                    s3 = TRUE)
  genes <- h5read(matrixh5_url,
                  name = "meta/genes",
                  s3 = TRUE)

  if (length(samples %in% GSM) == 0) {
    stop("The defined GSM ids do not have any match in ArchS4 database. \n We suggest to contact ArchS4 curator to add them.")
  }
  sample_locations <- which(samples %in% GSM)

  # extract gene expression from compressed data
  expression <- h5read(matrixh5_url, "data/expression",
                       index = list(seq_along(genes),
                                    sample_locations), s3 = TRUE)
  H5close()

  rownames(expression) <- genes

  colnames(expression) <- samples[sample_locations]

  # Print file
  if (print_tsv) {
    write.table(expression, file = filename, sep = "\t", quote = FALSE)
  }

  return(as.data.frame(expression))
}

---
title: "Introduction to SurfR"
author: "Aurora Maurizio, Anna Sofia Tascini, and Marco Jacopo Morelli"
output:
  BiocStyle::html_document:
    toc_float: true
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Introduction to SurfR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

Proteins at the cell surface connect intracellular and extracellular signaling networks and largely determine a cell’s capacity to communicate and interact with its environment. 

Importantly, variations in transcriptomic profiles are often observed between healthy and diseased cells, presenting distinct sets of cell-surface proteins. Indeed, cell surface proteins i) may act as biomarkers for the detection of diseased cells in tissues or body fluids and ii) are the most prevalent target of human drugs: 66% of approved human drugs listed in the DrugBank database target a cell-surface protein. The investigation of the cell surfaceome therefore could provide new possibilities for diagnosis, prognosis, treatment development, and therapy response evaluation.


When a study aims to find new biomarkers, the small number of samples often limits the ability to obtain reliable results. However, as sequencing costs continue to decrease, several follow-up experiments will likely be conducted to re-address the same biological question, suggesting a need for methods able to jointly analyze data from multiple studies.


**SurfR** provides a solution to these issues, generating a list of ranked surface protein-coding differentially-expressed genes starting from the raw count matrix of your own RNA-seq experiment or from bulk transcriptomic data automatically retrieved from public databases such as GEO and TCGA.
GEO queries are based on the ArchS4 pipeline. TCGA repository is interrogated through TCGAbiolinks.

**SurfR** also offers the opportunity to increase the sample size of a cohort by integrating related datasets, therefore enhancing the power to detect differentially expressed genes of interest. Meta-analysis can be performed through metaRNASeq, taking into account inter-study variability that may arise from technical differences among studies (e.g., sample preparation, library protocols, batch effects) as well as additional biological variability.

Gene ontology (GO) and pathway annotation can also be performed within **SurfR** to gain further insights about surface protein candidates.

Finally, **SurfR** includes functions to visualize DEG and enrichment results, aiding in the interpretation of findings.

## Citation

Maurizio, A., Tascini, A.S., Morelli, M. SurfR: Riding the wave of RNA-Seq data with a comprehensive Bioconductor package to identify Surface Protein Coding Genes. Bioinformatics Advances, 2024 (DOI: 10.1093/bioadv/vbae201)

# Installation

Install the package from *Bioconductor* or GitHub, ensuring correct
*Bioconductor* dependencies.

```{r install, eval = FALSE}
if (!"BiocManager" %in% rownames(installed.packages()))
    install.packages("BiocManager", repos = "https://cran.r-project.org")
```

When the package is available on *Bioconductor*, use

```{r install-Bioconductor, eval = FALSE}
BiocManager::install("SurfR")
```

Use the pre-release or devel version from GitHub using devtools with

```{r install-github, eval = FALSE}
# install.packages("devtools")
devtools::install_github("auroramaurizio/SurfR", build_vignettes = TRUE)
```

If you encounter issues during installation or execution, try downgrading dplyr to version 2.3.4.


# Quick Start

The basic idea behind **SurfR** has been to create a complete framework to detect surface protein coding genes within your data, or within public datasets. This framework facilitates the simultaneous analysis and comparison of multiple studies, easily revealing functional consensus and differences among distinct conditions. To begin, let’s ask **SurfR** to detect surface protein coding genes among a vector of input genes. Gene ID can be provided as  `gene_name`, `ensembl`, `entrez` or `uniProt_name`.

The protein classification is based on a recently developed surfaceome predictor, called [**SURFY**](http://wlab.ethz.ch/surfaceome/) [@bausch-fluck_silico_2018], based on machine learning.


```{r gene2protein, eval=FALSE}
library(SurfR)

GeneNames <- c("CIITA", "EPCAM", "CD24", "CDCP1", "LYVE1")
SurfaceProteins_df <- Gene2SProtein(GeneNames, 
                                    input_type = "gene_name", 
                                    output_tsv = FALSE,
                                    Surfy_version = "new")
#The output dataframe contains several information retrieved from Surfy.
colnames(SurfaceProteins_df)
#These are the 5 surface protein coding genes detected by SurfR.
SurfaceProteins_df$UniProt.gene
```



# Tutorial 

## Start from your own data

Although **SurfR** provides many functions to retrieve public data you can always start from your own dataset.

Here we are going to simulate a small bulkRNA dataset with the R package [SPsimSeq](https://doi.org/10.1093/bioinformatics/btaa105) [@assefa_spsimseq_2020]
starting from a subset of Zhang RNA-seq data [@zhang_targeting_2015], 
adding 20% of Differentially Expressed genes  (`pDE = 0.2`).

You can than decide to stick to it or combine it with other datasets (public or private).

```{r simulate zhang, eval=FALSE}
library(SPsimSeq)

data("zhang.data.sub")
zhang.counts <- zhang.data.sub$counts
MYCN.status  <- zhang.data.sub$MYCN.status

# Simulation of bulk RNA data
sim.data.bulk <- SPsimSeq(n.sim = 1, 
                          s.data = zhang.counts,
                          group = MYCN.status, 
                          n.genes = 1000, 
                          batch.config = 1,
                          group.config = c(0.5, 0.5), 
                          tot.samples = ncol(zhang.counts),
                          pDE = 0.2, 
                          lfc.thrld = 0.5, 
                          result.format = "list",
                          return.details = TRUE)

sim.data.bulk1 <- sim.data.bulk$sim.data.list[[1]]

countMatrix <- sim.data.bulk1$counts
row.names(countMatrix) <- row.names(zhang.counts)
metadata <- sim.data.bulk1$colData
metadata$Group <- as.factor(metadata$Group)
```

A fundamental task in the analysis of count data from RNA-seq is the detection of differentially expressed genes. For this task we rely on the package [DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html) [@love_moderated_2014], starting from counts data. The count data are presented as a table which reports, for each sample, the number of sequence fragments that have been assigned to each gene. 

We use the built-in SurfR **DGE** function, 
to perform the differential expression analysis of the simulated dataset.

Good Differential Expressed surface protein are the ones which are strongly expressed in one condition and almost not expressed in the other. To help detecting those candidates, the output data.frame of the DGE function contains information on the average expression in the two group (see Mean_CPM_T and Mean_CPM_C columns).

```{r DGE zhang, eval=FALSE}
library(SurfR)

df_zhang <- DGE(expression = countMatrix,
                metadata = metadata,
                Nreplica = 50,
                design = "~Group",
                condition = "Group",
                alpha = 0.05,
                TEST = "1", CTRL =  "0", 
                output_tsv = FALSE)

head(df_zhang)
```

Once DEGS have been detected, we may want to isolate Surface protein-coding genes.

The protein classification is based on a recently developed surfaceome predictor, 
called  [**SURFY**](http://wlab.ethz.ch/surfaceome/), based on machine learning. 

We use the built-in SurfR **Gene2SProtein** function to identify
Surface protein-coding genes (SP). 

```{r G2P zhang, eval=FALSE}
# remove NA values
df_zhang <- df_zhang[!is.na(df_zhang$padj), ]

# all fdr
fdr_GeneID <- df_zhang[df_zhang$padj < 0.05, "GeneID"]
SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")

# upregulated fdr
fdrUP_GeneID <- df_zhang[df_zhang$padj < 0.05 & df_zhang$log2FoldChange > 0,
                         "GeneID"]
SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")

# dowregulated fdr
fdrDW_GeneID <- df_zhang[df_zhang$padj < 0.05 & df_zhang$log2FoldChange < 0,
                         "GeneID"]
SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")
```



## Explore public datasets

### Dowload from GEO (Gene Expression Omnibus)

[GEO](https://www.ncbi.nlm.nih.gov/geo/) [@edgar_gene_2002] is a public functional genomics data repository containing high throughput gene expression data and hybridization arrays. 
We provide a handy interface to download experiments and curated gene expression profiles.

Here we are going to reanalyze the [Cloughesy et al.](https://doi.org/10.1038/s41591-018-0337-7) 
[@cloughesy_neoadjuvant_2019] public dataset of recurrent glioblastoma patients undergoing two different treatments: 
neoadjuvant pembrolizumab or adjuvant pembrolizumab.. 

This study is available under the GEO accession series [GSE121810](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE121810).

The metadata is downloaded with SurfR built-in function **GEOmetadata**. 
Note that this study has been sequenced with only one sequencing platform. If this is not the case, you have to download separately all the metadata specifying the GPL series numbers, and then merge them.

The count matrix is downloaded from [ArchS4](https://maayanlab.cloud/archs4/)
[@lachmann_massive_2018] with the SurfR built-in function **DownloadArchS4**,
to ensure handling not normalize data.

```{r GEO input meta_counts, eval=FALSE}
library(SurfR)
library(stringr)

# Download metadata from GEO
mGSE121810 <- GEOmetadata(GSE = "GSE121810")

# create new metadata column in order to remove unwanted special characters
unwanted_character <- " "
fx <- function(x) {
  str_split(string = x, pattern = unwanted_character)[[1]][1]
}
mGSE121810$condition <- sapply(mGSE121810$therapy, fx)
mGSE121810$condition <- as.factor(mGSE121810$condition)

# Preview metadata
head(mGSE121810)

# only select 3 samples per condition to save time
na_samples <- c("GSM3447013", "GSM3447018", "GSM3447019")
a_samples <- c("GSM3447023", "GSM3447024", "GSM3447026")
mGSE121810 <- mGSE121810[c(na_samples, a_samples), ]

# Download count matrix from ArchS4
cGSE121810 <- DownloadArchS4(mGSE121810$GSM, 
                             species = "human", 
                             print_tsv = FALSE, 
                             filename = NULL)

# Preview count matrix
head(cGSE121810[, ])
```

A fundamental objective in the analysis of RNA-seq counts data is the detection of differentially expressed genes. For this task we rely on the package DESeq2, starting from count data. Count data reports for each sample the number of sequence fragments that have been assigned to each gene.

Here, we use the built-in SurfR **DGE** function, to perform the differential expression analysis of the GSE121810 dataset.

Good Differential Expressed surface protein are the ones which are strongly expressed in one condition and almost not expressed in the other. To help detecting the best candidates, the output data.frame of the DGE function contains information on the average expression in the two group (see Mean_CPM_T and Mean_CPM_C columns).

```{r GEO dge, eval=FALSE}
# Perform DGE
df_GEO <- DGE(expression = cGSE121810,
              metadata = mGSE121810,
              Nreplica = 3,
              design = "~condition",
              condition = "condition",
              alpha = 0.05,
              TEST = "neoadjuvant", CTRL =  "adjuvant",
              output_tsv = FALSE)

# remove NA values
df_GEO <- df_GEO[!is.na(df_GEO$padj), ]

head(df_GEO)
```

Once DEGS have been detected, we may want to isolate Surface protein-coding genes.

The protein classification is based on a recently developed surfaceome predictor, 
called **SURFY**, based on machine learning. 

We use the built-in SurfR **Gene2SProtein** function to identify
Surface protein-coding genes (SP). 

```{r GEO SP, eval=FALSE}
# Detect SP amoung differentially expressed genes
fdr_GeneID <- df_GEO[df_GEO$padj < 0.1, "GeneID"]

SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")

fdrUP_GeneID <- df_GEO[df_GEO$padj < 0.1 & df_GEO$log2FoldChange > 0, "GeneID"]
SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")

fdrDW_GeneID <- df_GEO[df_GEO$padj < 0.1 & df_GEO$log2FoldChange < 0, "GeneID"]
SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")

```


### Download from TCGA (The Cancer Genome Atlas Program)

[TCGA](https://tcga-data.nci.nih.gov/tcga/) [@the_cancer_genome_atlas_research_network_cancer_2013] contains data for thousands of tumor samples across more than 20 types of cancer. 
Navigating through all of the files manually is impossible. Therefore we provide a function based on TCGAbiolinks that automates and streamlines the retrieval of public TCGA transcriptomics data. 
Note that to use this function you need to install the developmental version of
[TCGAbiolinks](https://github.com/BioinformaticsFMRP/TCGAbiolinks) [@mounir_new_2019] [@colaprico_tcgabiolinks_2016].

```{r TCGA install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BioinformaticsFMRP/TCGAbiolinksGUI.data")
BiocManager::install("BioinformaticsFMRP/TCGAbiolinks")
```

Here we reanalyze the *TCGA-THYM* dataset, since it is one of the smallest 
TCGA datasets, including normal solid tissue samples. 

The TCGA count matrix and the metadata are downloaded with SurfR built-in 
function **TCGA_download**. The shortLetterCode column of the metadata allows us 
to distinguish Primary Solid Tumor (TP) and normal (NT) samples.


```{r TCGA THYM Download, warning=FALSE, message = FALSE, eval=FALSE}
# Download TCGA data
# To save time only download 3 Tumor samples and 3 normal samples.
# If barcodes are not specified TCGA_download function will download all the 
# available samples in the TCGA study
barcodes2Download <- c("TCGA-X7-A8D6-11A-22R-A42C-07", 
                       "TCGA-X7-A8D7-11A-11R-A42C-07",
                       "TCGA-XM-A8RB-01A-11R-A42C-07", 
                       "TCGA-X7-A8M0-01A-11R-A42C-07")
TCGA.THYM <- TCGA_download(project = "TCGA-THYM", barcodes = barcodes2Download)

cTCGA.THYM <- TCGA.THYM[[1]]
mTCGA.THYM <- TCGA.THYM[[2]]
table(mTCGA.THYM$shortLetterCode)

mTCGA.THYM$shortLetterCode <- as.factor(mTCGA.THYM$shortLetterCode)
```


A fundamental task in the analysis of count data from RNA-seq is the detection of differentially expressed genes. For this task we rely on the package DESeq2, starting from counts data. The count data are presented as a table which reports, for each sample, the number of sequence fragments that have been assigned to each gene. 

We use the built-in SurfR **DGE** function to perform the differential expression 
analysis of the TCGA-THYM dataset.

Good Differential Expressed surface protein are the ones which are strongly expressed in one condition and almost not expressed in the other. To help detecting those candidates, the output data.frame of the DGE function contains information on the average expression in the two group (see Mean_CPM_T and Mean_CPM_C columns).

```{r TCGA DGE, eval=FALSE}
df_TCGA <- DGE(expression = cTCGA.THYM,
               metadata = mTCGA.THYM,
               Nreplica = 2,
               design = "~shortLetterCode",
               condition = "shortLetterCode",
               alpha = 0.05,
               TEST = "TP", CTRL =  "NT",
               output_tsv = FALSE)

head(df_TCGA)
```


Once DEGs have been detected, we may want to isolate Surface protein-coding genes.

The protein classification takes advantage of a recently developed surfaceome predictor, 
called **SURFY**, based on machine learning. 

We use the built-in SurfR **Gene2SProtein** function to identify
Surface protein-coding genes (SP).

```{r TCGA SP, eval=FALSE}
# remove NA values
df_TCGA <- df_TCGA[!is.na(df_TCGA$padj), ]

fdr_GeneID <- df_TCGA[df_TCGA$padj < 0.05,
                      "GeneID"]

SP <- Gene2SProtein(genes = fdr_GeneID, input_type = "gene_name")

fdrUP_GeneID <- df_TCGA[df_TCGA$padj < 0.05 & df_TCGA$log2FoldChange > 0,
                        "GeneID"]
SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")

fdrDW_GeneID <- df_TCGA[df_TCGA$padj < 0.05 & df_TCGA$log2FoldChange < 0,
                        "GeneID"]
SPdw <- Gene2SProtein(genes = fdrDW_GeneID, input_type = "gene_name")
```



## Meta-analysis

Analyzing data arising from several experiments studying the same question is a 
way to obtain more robust results, increasing the detection power of differentially expressed genes.

In SurfR we provide a set of functions based on [MetaRNASeq](https://cran.r-project.org/web/packages/metaRNASeq/vignettes/metaRNASeq.pdf) 
[@rau_differential_2014] package to combine data from multiple RNAseq experiments.

Let’s suppose we want to integrate Breast cancer data from GEO and TCGA. 
Breast Cancer datasets are downloaded from TCGA with the SurfR built-in function  **TCGA_download**.


```{r TCGA BRCA Download, warning=FALSE, message = FALSE, eval=FALSE}
# Download TCGA data
# To save time only download 3 Tumor samples and 3 normal samples.
# If barcodes are not specified TCGA_download function will download all the 
# available samples in the TCGA study
barcodes2Download <- c("TCGA-BH-A1FU-11A-23R-A14D-07", 
                       "TCGA-BH-A1FC-11A-32R-A13Q-07",
                       "TCGA-BH-A0DO-11A-22R-A12D-07", 
                       "TCGA-B6-A0RH-01A-21R-A115-07",
                       "TCGA-BH-A1FU-01A-11R-A14D-07", 
                       "TCGA-A1-A0SE-01A-11R-A084-07")
TCGA.BRCA <- TCGA_download(project = "TCGA-BRCA", barcodes = barcodes2Download)
```

```{r meta TCGA_input, eval=FALSE}
cTCGA.BRCA <- TCGA.BRCA[[1]]
mTCGA.BRCA <- TCGA.BRCA[[2]]
table(mTCGA.BRCA$shortLetterCode)

```

In GEO, we want to analyze [Varley](https://doi.org/10.1007/s10549-014-3019-2) data 
[@varley_recurrent_2014], which includes samples of ER+ breast cancer, Triple Negative Breast cancer, 
adjacent tissues, and normal breast. These datasets can be retrieved from the GEO series [GSE58135](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE58135), 
using the SurfR built-in functions **GEOmetadata** and **DownloadArchS4**.

```{r meta GEO_input metadata, eval = FALSE}
mGSE58135 <- GEOmetadata("GSE58135")
mGSE58135 <- mGSE58135[mGSE58135$tissue != "Breast Cancer Cell Line", ]
mGSE58135$condition <- "NT"
mGSE58135$condition[mGSE58135$tissue %in% c("ER+ Breast Cancer Primary Tumor", 
                                            "Triple Negative Breast Cancer Primary Tumor")] <- "TP"

# only select 3 samples per condition to save time
TP_samples <- c("GSM1401694", "GSM1401717", "GSM1401729")
NT_samples <- c("GSM1401799", "GSM1401813", "GSM1401814")
mGSE58135 <- mGSE58135[c(TP_samples, NT_samples), ]
```

```{r meta GEO_input counts, eval = FALSE}
cGSE58135 <- DownloadArchS4(mGSE58135$GSM, species = "human")
cGSE58135 <- cGSE58135[, row.names(mGSE58135)]

table(mGSE58135$condition)

```

The first step in the analysis is to detect differentially expressed genes for each count data, separately. 
For this task we rely on the package DESeq2, starting from counts data. The count data are presented as a table which reports, for each sample, the number of sequence fragments that have been assigned to each gene.

We use the built-in SurfR **DGE** function to perform the differential expression 
analysis of the TCGA-BRCA dataset and GSE58135.

Good differentially expressed surface proteins should be strongly 
expressed in one condition and almost not expressed in the other. The output dataframe of the DGE function contains information 
on the average expression in the two groups (see Mean_CPM_T and Mean_CPM_C columns), to help in the detection of the best candidates.


```{r meta DGE TCGA, eval=FALSE}
# TCGA DGE
df.TCGA <- DGE(expression = cTCGA.BRCA,
               metadata = mTCGA.BRCA,
               Nreplica = 3,
               design = "~shortLetterCode",
               condition = "shortLetterCode",
               alpha = 0.05,
               TEST = "TP", CTRL =  "NT",
               output_tsv = FALSE)
head(df.TCGA)
```

```{r meta DGE GEO, eval = FALSE}
# GSE58135 DGE
df.GSE58135 <- DGE(expression = cGSE58135,
                   metadata = mGSE58135,
                   Nreplica = 3,
                   design = "~condition",
                   condition = "condition",
                   alpha = 0.05,
                   TEST = "TP", CTRL =  "NT",
                   output_tsv = FALSE)
head(df.GSE58135)
```

Here we provide a function based on **metaRNASeq** bioconductor package to implement 
two p-value combination techniques (inverse normal and Fisher methods).

The meta-analysis is performed by the SurfR built-in function **metaRNAseq**, 
which requires as input: 

* a list of data.frames with the DGE results of the chosen databases to combine (`ind_deg`); 
* the statistic test to use to combine p.values, which can be the Fisher method (`fishercomb`)
or the inverse normal combination technique (`invnorm`);
* the Benjamini Hochberg threshold (`BHth`);
* if using inverse normal combination technique, a vector of the number of replicates used in each study to calculate the previous one-sided p-values (`nrep`).

The function automatically produces and saves as .pdf histograms of raw p-values for each of the individual differential analyses performed using the independent filtering from DESeq2 package. You can also examine the p-value distribution after p.value combination.


```{r meta fishercomb, eval=FALSE}

L_fishercomb <- metaRNAseq(ind_deg = list(TCGA.BRCA =  df.TCGA, GEO.GSE58135 = df.GSE58135),
                           test_statistic = "fishercomb",
                           BHth = 0.05,
                           adjpval.t = 0.05)
```

```{r meta invnorm, eval=FALSE}
L_invnorm <- metaRNAseq(ind_deg = list(TCGA.BRCA =  df.TCGA, GEO.GSE58135 = df.GSE58135),
                        test_statistic = "invnorm",
                        BHth = 0.05,
                        adjpval.t = 0.05,
                        nrep = c(102, 56))
```

Finally, we can summarize the results of the meta-analysis in a data.frame highlighting the statistical information for the common genes to all methods using the built-in SurfR function **combine_fisher_invnorm** and use the built-in SurfR **Gene2SProtein** function to identify Surface protein-coding genes (SP) among those.

Genes displaying contradictory differential expression in separate studies can be identified in the column `signFC`= 0 and removed from the list of differentially expressed genes via meta-analysis.


```{r metacombine, eval=FALSE}
metacomb <- combine_fisher_invnorm(ind_deg = list(TCGA.BRCA =  df.TCGA, 
                                                  GEO.GSE58135 = df.GSE58135),
                                   invnorm = L_invnorm,
                                   fishercomb = L_fishercomb,
                                   adjpval = 0.05)


metacomb_GeneID <- metacomb[metacomb$signFC != 0, "GeneID"]
SP <- Gene2SProtein(genes = metacomb_GeneID, input_type = "gene_name")
```

```{r metacombine up, eval=FALSE}
metacombUP_GeneID <- metacomb[metacomb$signFC == 1, "GeneID"]
SPup <- Gene2SProtein(genes = metacombUP_GeneID, input_type = "gene_name")
```

```{r metacombine dw, eval=FALSE}
metacombDW_GeneID <- metacomb[metacomb$signFC == -1, "GeneID"]
SPdw <- Gene2SProtein(genes = metacombDW_GeneID, input_type = "gene_name")

```



## Functional Enrichment
After identifying the subset of genes enriched in our specific condition of interest, 
a range of analyses becomes useful to move beyond a mere gene list.

A general enrichment analysis allows to gain further insights about upregulated or downregulated DEGs. 
To do so, we use the SurfR built-in function **Enrichment**,  based on the [enrichR](https://cran.r-project.org/web/packages/enrichR/vignettes/enrichR.html) cran package [@kuleshov_enrichr_2016]. 
You have the option to indicate the specific database you wish to utilize among the available in enrichR. 
The enrichR function `listEnrichrDbs()` allows you to navigate the options. 
Sporadically, network connectivity issues may arise with EnrichR server. 
If it happens, please, retry to run the function after a few minutes.



```{r enrich, eval=FALSE}

dfList <- list(GEO = df.GSE58135,
               TCGA = df.TCGA)

# Enrichment analysis
Enrich <- Enrichment(dfList, 
                     enrich.databases = c("GO_Biological_Process_2021", 
                                          "KEGG_2021_Human"),
                     p_adj = 0.05, logFC = 1)

head(Enrich$GEO$fdr_up$GO_Biological_Process_2021)
```

**SurfR** implements several visualization methods to help interpret enrichment results 
obtained through EnrichR using ggplot2, with the built-in function 
**Enrichment_barplot**.

It depicts gene count ratio and enrichment scores (- Log10 adjusted p values) as bar height and color. Users can specify the number of terms (most significant) to display.

```{r, fig.width=7, fig.height=2.5, eval=FALSE}
library(ggplot2)

# barplot of the top 5 upregulated pathways
Enrichment_barplot(Enrich$GEO, 
                   enrich.databases <- c("GO_Biological_Process_2021",  
                                         "KEGG_2021_Human"), 
                   p_adj = 0.05, 
                   num_term = 5, 
                   cond = "UP")

# barplot of the top 5 downregulated pathways
Enrichment_barplot(Enrich$GEO, 
                   enrich.databases <- c("GO_Biological_Process_2021", 
                                         "KEGG_2021_Human"), 
                   p_adj = 0.05, 
                   num_term = 5, 
                   cond = "DOWN")
```


Moreover, we can annotate our list of genes with cross-database identifiers and descriptions (Entrezid, Uniprot, KEGG, etc.),
taking advantage of one of the 35 gene-set libraries present in the Enrichr database, 
using the SurfR built-in function **Annotate_SPID**. 

```{r anno spid, eval=FALSE}
annotated <- Annotate_SPID(df.GSE58135, "WikiPathway_2021_Human") 
head(annotated, 10)
```

## Results visualization

### Bar plot 

Utilizing the SurfR function **Splot** you can create barplots to visualize the annotation classes reported in
the dataframe produced by the **Gene2SProtein** function.
The default grouping is the [Membranome Almen classification](https://doi.org/10.1186/1741-7007-7-50). 

```{r, fig.height= 4.5, fig.width=7, eval = FALSE}
# upregulated genes in GEO dataset
fdrUP_GeneID <- df.GSE58135[df.GSE58135$padj < 0.05 & df.GSE58135$log2FoldChange > 0, "GeneID"]
# corresponding Surface Proteins
SPup <- Gene2SProtein(genes = fdrUP_GeneID, input_type = "gene_name")
# Barplot of Almen classification 
Splot(SPup,
      group.by = "Membranome.Almen.main-class",
      main = "Almen class")
```

### Venn diagram

You can compare the list of resulting surface proteins from up to 7 different studies, 
using a venn diagram,
with the built-in SurfR function **SVenn**.


```{r, fig.height= 6, fig.width=7, eval = FALSE}

S_list <- list(SP_1 = c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1"),
               SP_2 = c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"))

SVenn(S_list,
      cols.use = c("green", "blue"),
      opacity = 0.5,
      output_intersectionFile = FALSE)

```


### PCA

Principal Components Analysis (PCA) is a very useful diagnostic feature to gain insights about your datasets.

You can perform PCA and visualize the result with a customizable plot 
with the built-in SurfR function **plotPCA**.


```{r, fig.height= 6, fig.width=7, eval = FALSE}

SurfR::plotPCA(matrix = edgeR::cpm(cGSE58135), metadata = mGSE58135,               
               dims = c(1, 2),
               color.by = "condition", shape.by = "condition", 
               label = FALSE, main = "PCA GSE58135")
```


# SessionInfo

```{r}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DownloadArchS4.R
\name{DownloadArchS4}
\alias{DownloadArchS4}
\title{DownloadArchS4 function}
\usage{
DownloadArchS4(GSM, species, print_tsv = FALSE, filename = NULL)
}
\arguments{
\item{GSM}{Vector with the GSM ids of the samples to consider.}

\item{species}{Specify the specie of yuor GSM samples. Either human or mouse.}

\item{print_tsv}{Logical. If \code{TRUE}, outputs a tsv file with the count matrix. By default, FALSE.}

\item{filename}{Name of the tsv output file. Default is matrix.tsv.}
}
\value{
A count matrix with gene on the row and GSM ID on the column.
}
\description{
Download count matrix from \url{https://maayanlab.cloud/archs4/},
given a vector of input GEO Sample accessions numbers (GSM).
}
\section{Warning}{

If the defined GSM ids do not have any match in ArchS4 database, we suggest to contact ArchS4 curator to add them.
}

\examples{
\dontrun{
GSM <- c("GSM3447008", "GSM3447009")
GEO_count_matrix <- DownloadArchS4(GSM, species = "human",
                                  print_tsv = FALSE, filename = NULL)}
}
\seealso{
\code{\link{GEOmetadata}} function for downloading GEO metadata.
\url{https://www.ncbi.nlm.nih.gov/geo} for info on GSM.
\url{https://maayanlab.cloud/archs4/} for info on ArchS4.

Other public-data functions: 
\code{\link{GEOmetadata}()},
\code{\link{TCGA_download}()}
}
\concept{public-data functions}

\name{countData}
\alias{countData}
\title{countData}
\value{
A dataframe.
}
\description{
  Simulated raw counts to use as input for DGE and plotPCA functions. metadata is available.
}
\usage{
  data(countData)
}
\format{
  dataframe
}
\details{
  A dataframe with 2500 rows and 4 columns (sample names).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Annotate_SPID.R
\name{.format_str}
\alias{.format_str}
\title{Format a string using placeholders - function from hypeR}
\usage{
.format_str(string, ...)
}
\arguments{
\item{string}{A an unformatted string with placeholders}

\item{...}{Variables to format placeholders with}
}
\value{
A formatted string
}
\description{
Format a string using placeholders - function from hypeR
}
\examples{
\dontrun{
format_str("Format with {1} and {2}", "x", "y")
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment.R
\name{getEnrichr}
\alias{getEnrichr}
\title{Check if EnrichR is online}
\usage{
getEnrichr(url, ...)
}
\arguments{
\item{url}{(Required). URL address to check}

\item{...}{(Optional). Additional parameters to pass to GET}
}
\value{
Invisible response object from GET request if successful, or NULL on failure
}
\description{
Helper function
}
\details{
Helper function for HTTP methods GET
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SVenn.R
\name{SVenn}
\alias{SVenn}
\title{SVenn}
\usage{
SVenn(
  S_list,
  cols.use = NULL,
  opacity = 0.5,
  output_intersectionFile = TRUE,
  filename = "intersection.xlsx"
)
}
\arguments{
\item{S_list}{A list of a maximum of 7 surface protein sets detected in different studies.}

\item{cols.use}{Vector of colors, each color corresponds to a study.
By default, ggplot assigns colors.}

\item{opacity}{Degree of opacity for the colors specified with cols.use (less opacity, more transparency).}

\item{output_intersectionFile}{logical. If TRUE (default) write an xlsx output of protein in the intersections.}

\item{filename}{Name of the output file with the intersections.}
}
\value{
venn plot of common genes.
}
\description{
Venn diagram of common surface proteins overexpressed among up to 7 different studies
}
\examples{
\dontrun{
S_list <- list(SP1 <- c("EPCAM", "CD24",  "DLK1",  "CDCP1", "LYVE1"),
              SP2 <- c("DLK1", "EPCAM", "EGFR", "UPK1A", "UPK2"))
SP <- SVenn(S_list, cols.use = c("pink", "yellow"), output_intersectionFile = FALSE)}
}
\seealso{
\code{\link{Gene2SProtein}} for detection of Surface proteins from a list of genes.

Other plot functions: 
\code{\link{Enrichment_barplot}()},
\code{\link{Splot}()},
\code{\link{plotPCA}()}
}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment.R
\name{Enrichment}
\alias{Enrichment}
\title{Enrichment function}
\usage{
Enrichment(
  dfList,
  enrich.databases = c("GO_Biological_Process_2021", "GO_Cellular_Component_2021",
    "GO_Molecular_Function_2021", "KEGG_2021_Human", "MSigDB_Hallmark_2020",
    "WikiPathways_2016", "BioCarta_2016", "Jensen_TISSUES", "Jensen_COMPARTMENTS",
    "Jensen_DISEASES"),
  p_adj = 0.05,
  logFC = 1,
  save.results = FALSE
)
}
\arguments{
\item{dfList}{Dataframes list}

\item{enrich.databases}{Vector of EnrichR databases to consult}

\item{p_adj}{Double. Adjusted pvalue threshold for the enrichment}

\item{logFC}{Double. Fold change threshold for the enrichment}

\item{save.results}{Logical. If TRUE saves input gene lists and enrichment results.}
}
\value{
A list of enrichment tables for upregulated and downregulated genes
in the different enrichr databases
}
\description{
Perform enrichment Analysis of RNA-Seq Data
}
\examples{
\dontrun{
df1 <- data.frame(GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5"),
                  baseMean = c(13490.22, 10490.23, 8888.33, 750.33),
                  log2FoldChange = c(5.78, 6.76, -7.78, -8.78),
                  padj = c(2.28e-143, 2.18e-115, 2.18e-45, 0.006),
                  row.names = c("MEST", "CDK1", "PCLAF", "BIRC5"))
df2 <- data.frame(GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5"),
                  baseMean = c(13490.22, 10490.23, 8888.33, 750.33),
                  log2FoldChange = c(5.78, 6.76, -7.78, -8.78),
                  padj = c(2.28e-143, 2.18e-115, 2.18e-45, 0.006),
                  row.names = c("MEST", "CDK1", "PCLAF", "BIRC5"))
dfList <- list(df1 = df1, df2 = df2)
test <- Enrichment(dfList, enrich.databases = c("GO_Cellular_Component_2021"),
                   save.results = FALSE)}
}
\seealso{
\url{https://maayanlab.cloud/Enrichr/} for additional information about enrichR.

Other functional-annotation functions: 
\code{\link{Annotate_SPID}()},
\code{\link{Enrichment_barplot}()}
}
\concept{functional-annotation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Splot.R
\name{Splot}
\alias{Splot}
\title{Splot function}
\usage{
Splot(
  SurfaceProteins_df,
  group.by = "Membranome.Almen.main-class",
  cols.use = NULL,
  main = "Almen main class"
)
}
\arguments{
\item{SurfaceProteins_df}{Output dataframe of Gene2SProtein function.}

\item{group.by}{Name of columns to plot. Default = Membranome.Almen.main-class.}

\item{cols.use}{Vector of colors, each color corresponds to an identity class.
By default, ggplot assigns colors.}

\item{main}{Plot title. Default = Almen main class.}
}
\value{
plot objec created by ggplot2,
which can be assigned and further customized.
}
\description{
Plot a barplot with features of Surface Protein
}
\examples{
\dontrun{
GeneNames <- c("CIITA", "EPCAM", "DLK1", "CD24", "CDCP1", "LYVE1", "ABCD1", "VAMP1")
SurfaceProteins_df <- Gene2SProtein(GeneNames, input_type = "gene_name")
Splot(SurfaceProteins_df)}
}
\seealso{
Other plot functions: 
\code{\link{Enrichment_barplot}()},
\code{\link{SVenn}()},
\code{\link{plotPCA}()}
}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Annotate_SPID.R
\name{enrichr_connect}
\alias{enrichr_connect}
\title{Connect to the enrichr web application  - function from hypeR}
\usage{
enrichr_connect(endpoint, db = c("Enrichr"))
}
\arguments{
\item{endpoint}{The url endpoint to connect to}

\item{db}{A species}
}
\value{
A web response
}
\description{
Connect to the enrichr web application  - function from hypeR
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaRNAseq.R
\name{metaRNAseq}
\alias{metaRNAseq}
\title{metaRNAseq function}
\usage{
metaRNAseq(
  ind_deg,
  test_statistic = "fishercomb",
  BHth = 0.05,
  adjpval.t = 0.05,
  nrep = NULL,
  plot = FALSE
)
}
\arguments{
\item{ind_deg}{List of indipendent named DEG dataframes with p-values to be combined.}

\item{test_statistic}{p-value combination technique (inverse normal or Fisher):
\code{fishercomb}, \code{invnorm}.
By default: \code{fishercomb}.}

\item{BHth}{Benjamini Hochberg threshold.}

\item{adjpval.t}{threshold to represent as binary the Meta-Analysis output adjpval.}

\item{nrep}{Vector of numbers of replicates used in each study to calculate the previous one-sided p-values.}

\item{plot}{Logical. If TRUE plot histogram of pvalues.
By default, the False Discovery Rate is controlled at 0.05.}
}
\value{
A list with \code{DEindices} of DEG at the chosen Benjamini Hochberg threshold, and
\code{TestStatistic}, \code{rawpval}, \code{adjpval}, \code{binaryadjpval} vectors for differential expression in the meta-analysis.
}
\description{
Perform Meta-Analysis of RNA-Seq Data
}
\examples{
\dontrun{
# Deseq2 output samples
DGE1 <- data.frame(GeneID = c("DLK1", "EPCAM"),
                 Mean_CPM_T = c(5.92, 9.91),
                 Mean_CPM_C = c(0.04, 0.03),
                 log2FoldChange = c(10.22, 8.42),
                 lfcSE = c(0.80, 0.48),
                 stat = c(12.68, 17.69),
                 pvalue = c(7.30135e-37, 4.37011e-70),
                 padj = c(1.49936e-35, 1.12976e-67),
                 row.names = c("DLK1", "EPCAM"))
DGE2 <- data.frame(GeneID = c("DLK1", "EPCAM"),
                 Mean_CPM_T = c(3.92, 8.91),
                 Mean_CPM_C = c(0.04, 0.03),
                 log2FoldChange = c(7.22, 5.81),
                 lfcSE = c(0.80, 0.48),
                 stat = c(12.68, 17.69),
                 pvalue = c(7.30135e-37, 4.37011e-70),
                 padj = c(1.49936e-35, 1.12976e-67),
                 row.names = c("DLK1", "EPCAM"))
# input list
ind_deg <- list(DEG1_df = DGE1, DEG2_df = DGE2)
# perform meta-analysis
comb_pval_df <- metaRNAseq(ind_deg, test_statistic = "invnorm", BHth = 0.05, nrep = c(2,2))}
}
\seealso{
\code{\link{DGE}} for DGE analysis,
and \url{https://cran.r-project.org/web/packages/metaRNASeq/vignettes/metaRNASeq.pdf}
for metaRNASeq package info.

Other meta-analysis functions: 
\code{\link{combine_fisher_invnorm}()}
}
\concept{meta-analysis functions}

\name{metadata}
\alias{metadata}
\title{metadata}
\value{
A dataframe.
}
\description{
  Metadata associated with countData for testing purposes (functions DGE, plotPCA).
}
\usage{
  data(metadata)
}
\format{
  dataframe.
}
\details{
  A dataframe with 4 rows (sample names) and 3 columns (samplesID, condition A and B, therapy T1 and T2).
}

\name{enrichedList}
\alias{enrichedList}
\title{enrichedList}
\value{
  A list of lists.
}
\description{
  Input list for Enrichment_barplot function. enrichedList is the output of
  Enrichment function applied to ind_deg object when enrich.databases is equal
  to GO_Cellular_Component_2021, default parameters.
}
\usage{
  data(enrichedList)
}
\format{
  list
}
\details{
  enrichedList$fdr_up$GO_Cellular_Component_2021 contains upregulated gene
  enrichments, enrichedList$fdr_down$GO_Cellular_Component_2021 contains
  downregulated gene enrichments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Annotate_SPID.R
\name{Annotate_SPID}
\alias{Annotate_SPID}
\title{Annotate_SPID}
\usage{
Annotate_SPID(
  DGE,
  enrich.database = "WikiPathway_2021_Human",
  output_tsv = FALSE
)
}
\arguments{
\item{DGE}{Data.frame containing annotated DEG list, as the output of DGE or Gene2SProtein functions.}

\item{enrich.database}{String containing the EnrichR databases you would like to consult. Default: WikiPathway_2021_Human.}

\item{output_tsv}{Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.}
}
\value{
A dataframe with surface protein coding \code{DEGs} annotation.
}
\description{
Annotate Surface Protein Coding genes according to EnrichR libraries
}
\section{Warning}{

Be sure that enrich.database exists.
}

\examples{
\dontrun{
# Deseq2 output sample
DGE = data.frame(GeneID = c("DLK1", "TOP2A"),
                 Mean_CPM_T = c(5.92, 9.91),
                 Mean_CPM_C = c(0.04, 0.03),
                 log2FoldChange = c(10.22, 8.42),
                 lfcSE = c(0.80, 0.48),
                 stat = c(12.68, 17.69),
                 pvalue = c(7.30135e-37, 4.37011e-70),
                 padj = c(1.49936e-35, 1.12976e-67))
annotated_DGE = Annotate_SPID(DGE, "WikiPathway_2021_Human")

# Output of Gene2SProtein function
GeneNames = c("CIITA", "EPCAM", "DLK1", "CD24")
SurfaceProteins_df = Gene2SProtein(GeneNames, input_type = "gene_name")
annotated_SP = Annotate_SPID(SurfaceProteins_df, "GO_Biological_Process_2021")}
}
\seealso{
\code{\link{DGE}} function for DGE,
and \code{\link{Gene2SProtein}} function for Gene2SProtein analysis

Other functional-annotation functions: 
\code{\link{Enrichment_barplot}()},
\code{\link{Enrichment}()}
}
\concept{functional-annotation functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Annotate_SPID.R
\name{enrichr_urls}
\alias{enrichr_urls}
\title{Get url base for species-specific enrichr libraries  - function from hypeR}
\usage{
enrichr_urls(db = c("Enrichr"))
}
\arguments{
\item{db}{A species}
}
\value{
A url
}
\description{
Get url base for species-specific enrichr libraries  - function from hypeR
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TCGA_download.R
\name{TCGA_download}
\alias{TCGA_download}
\title{TCGA_download function}
\usage{
TCGA_download(
  project,
  whichcounts = "unstranded",
  save.matrix = FALSE,
  save.metadata = FALSE,
  barcodes = NULL
)
}
\arguments{
\item{project, }{Character. A valid project from TCGAbiolinks:::getGDCprojects()$project_id}

\item{whichcounts}{Character. Counts data to use. Choose from: unstranded, stranded_first,stranded_second. By default, unstranded.}

\item{save.matrix}{Logical. If \code{TRUE}, outputs a tsv file with the Matrix. By default, FALSE.}

\item{save.metadata}{Logical. If \code{TRUE}, outputs a tsv file with the metadata. By default, FALSE.}

\item{barcodes}{Character. A vector with names of the barcodes you want to download. If NULL (default) it downloads all the available barcodes in the project.}
}
\value{
A list containing the Matrix and the metadata.
}
\description{
Downloads count matrix data from TCGA
}
\examples{
\dontrun{
GBM_list_s1 <- TCGA_download(project="TCGA-GBM",
                             whichcounts = "unstranded",
                             save.matrix = FALSE, save.metadata = FALSE,
                             barcodes = c("TCGA-06-0878-01A-01R-1849-01"))
remove downloaded data from TCGA
unlink('GDCdata', recursive = TRUE, force = TRUE)
file.remove("MANIFEST.txt")
}
}
\seealso{
Other public-data functions: 
\code{\link{DownloadArchS4}()},
\code{\link{GEOmetadata}()}
}
\concept{public-data functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DGE.R
\name{DGE}
\alias{DGE}
\title{DGE function}
\usage{
DGE(
  expression,
  metadata,
  Nreplica,
  design = "~condition",
  condition = "condition",
  TEST,
  CTRL,
  alpha = 0.05,
  FC_filt = 0,
  output_tsv = FALSE,
  output_filename = "DEGs.tsv"
)
}
\arguments{
\item{expression}{Dataframe with counts}

\item{metadata}{Dataframe with sample metadata}

\item{Nreplica}{Double. Minimum number of replicates in each group}

\item{design}{Design formula for DGE}

\item{condition}{Column of the metadata ti use for DGE results}

\item{TEST}{Character. sample name in metadata}

\item{CTRL}{Character. sample name in metadata}

\item{alpha}{Double. the significance cutoff used for optimizing the independent filtering (by default 0.1).
If the adjusted p-value cutoff (FDR) will be a value other than 0.1, alpha should be set to that value.}

\item{FC_filt}{Dataframe with counts}

\item{output_tsv}{Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.}

\item{output_filename}{Name of the tsv output file. Default is DEGs.tsv.}
}
\value{
A dataframe with \code{DEGs}
}
\description{
Perform Differential Gene Expression Analysis of RNA-Seq Data
}
\examples{
\dontrun{
# Simulation of bulk RNA data
countData <- matrix(floor(runif(10000, min=0, max=101)),ncol=4)
colnames(countData) <- paste("sample", seq_len(ncol(countData)), sep = "")
rownames(countData) <- paste("gene", seq_along(seq_len(10000/4)), sep = "")
metadata <- data.frame(samplesID = paste("sample", seq_len(ncol(countData)), sep = ""),
                       condition = factor(c("A","A","B","B")))
row.names(metadata) <- metadata$samplesID
# Perform DGE
DGEresults <- DGE(expression = countData, metadata = metadata,
                 Nreplica = 2,
                 design = "~condition",condition = "condition",
                 TEST = "A", CTRL = "B")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment.R
\name{setEnrichrSite}
\alias{setEnrichrSite}
\title{Set Enrichr Website}
\usage{
setEnrichrSite(site)

setEnrichrSite(site)
}
\arguments{
\item{site}{site requested}
}
\value{
Changes Enrichr Website connection

Changes Enrichr Website connection
}
\description{
Set Enrichr Website

Set Enrichr Website
}
\details{
Set Enrichr Website

Set Enrichr Website
}
\author{
Alexander Blume
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment.R
\name{listEnrichrDbs}
\alias{listEnrichrDbs}
\title{Look up available databases on Enrichr}
\usage{
listEnrichrDbs()
}
\value{
A data.frame of available Enrichr databases
}
\description{
Look up available databases on Enrichr
}
\details{
Look up available databases on Enrichr
}
\examples{
dbs <- listEnrichrDbs()
}
\author{
Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gene2SProtein.R
\name{Gene2SProtein}
\alias{Gene2SProtein}
\title{Gene2SProtein function}
\usage{
Gene2SProtein(
  genes,
  input_type = "gene_name",
  output_tsv = FALSE,
  output_filename = "surfaceProteins.tsv",
  Surfy_version = "log"
)
}
\arguments{
\item{genes}{A vector of genes.}

\item{input_type}{The gene identification type:
\code{gene_name}, \code{ensembl}, \code{entrez} or \code{uniProt_name}.
By default: \code{gene_name}.}

\item{output_tsv}{Logical. If \code{TRUE}, outputs a tsv file with the results. By default, FALSE.}

\item{output_filename}{Name of the tsv output file. Default is surfaceProteins.tsv.}

\item{Surfy_version}{The version of surfy dataframe you wish to use. Choose between \code{log} or \code{newest}.
By default use the most recent \code{log} version.
If a log dataframe does not exist the \code{newest} is downloaded from \url{https://wlab.ethz.ch/surfaceome}.}
}
\value{
A data frame with filtered surface proteins from the \code{genes} array.
The dataframe contains also addition information obtained from surfy.
}
\description{
Detect Surface Proteins from a vector of genes.
The surface proteins are identified according to the in silico human surfaceome database,
available at \url{https://wlab.ethz.ch/surfaceome}.
}
\section{Warning}{

The surfy database is interrogated using the gene identification type of your preference
between \code{gene_name}, \code{ensembl}, \code{entrez} or \code{uniProt_name}. Note that
you might loose some matches due to different gene version IDs.
}

\examples{
 \dontrun{
 # from gene name IDs to Surface proteins
 GeneNames <- c("CIITA", "EPCAM", "DLK1", "CD24", "CDCP1", "LYVE1", "ABCD1", "VAMP1")
 SurfaceProteins_df <- Gene2SProtein(GeneNames, input_type = "gene_name")

 # from ensembl IDs to Surface proteins
 Ensembl <- c("ENSG00000178343", "ENSG00000176895", "ENSG00000162419", "ENSG00000170776",
             "ENSG00000092529", "ENSG00000135926", "ENSG00000152595", "ENSG00000121577",
             "ENSG00000186094", "ENSG00000126773", "ENSG00000198918", "ENSG00000167378",
             "ENSG00000095574", "ENSG00000140678", "ENSG00000262484", "ENSG00000133739",
             "ENSG00000172469", "ENSG00000112992", "ENSG00000148343", "ENSG00000138593")
 SurfaceProteins_df <- Gene2SProtein(Ensembl, input_type = "ensembl",
                                   output_tsv = FALSE, Surfy_version = "new")}
}
\seealso{
\code{\link{DGE}} for DGE analysis,
\url{https://wlab.ethz.ch/surfaceome} for info on Surfy
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combine_fisher_invnorm.R
\name{combine_fisher_invnorm}
\alias{combine_fisher_invnorm}
\title{combine_fisher_invnorm}
\usage{
combine_fisher_invnorm(
  ind_deg,
  invnorm,
  fishercomb,
  adjpval = 0.05,
  output_tsv = TRUE,
  output_filename = "combine_fisher_invnorm.tsv"
)
}
\arguments{
\item{ind_deg}{List of indipendent DEG dataframes with p-values to be combined.}

\item{invnorm}{inverse normal p-value combination technique dataframe (output of metaRNAseq)}

\item{fishercomb}{Fisher p-value combination technique dataframe (output of metaRNAseq)}

\item{adjpval}{threshold to represent as binary the Meta-Analysis output adjpval.}

\item{output_tsv}{logical. If TRUE, it outputs table with results. Default: TRUE}

\item{output_filename}{File name for the results file.}
}
\value{
A dataframe with \code{DEindices} and \code{DEname} of DEG at the chosen Benjamini Hochberg threshold, and
\code{TestStatistic}, \code{rawpval}, \code{adjpval}, \code{binaryadjpval} vectors for differential expression in the meta-analysis.
}
\description{
Combine Meta-Analysis results with individual DE tables
}
\examples{
\dontrun{
# Deseq2 output samples
DGE1 <- data.frame(GeneID = c("DLK1", "EPCAM"),
                 Mean_CPM_T = c(5.92, 9.91),
                 Mean_CPM_C = c(0.04, 0.03),
                 log2FoldChange = c(10.22, 8.42),
                 lfcSE = c(0.80, 0.48),
                 stat = c(12.68, 17.69),
                 pvalue = c(7.30135e-37, 4.37011e-70),
                 padj = c(1.49936e-35, 1.12976e-67),
                 row.names = c("DLK1", "EPCAM"))
DGE2 <- data.frame(GeneID = c("DLK1", "EPCAM"),
                 Mean_CPM_T = c(3.92, 8.91),
                 Mean_CPM_C = c(0.04, 0.03),
                 log2FoldChange = c(7.22, 5.81),
                 lfcSE = c(0.80, 0.48),
                 stat = c(12.68, 17.69),
                 pvalue = c(7.30135e-37, 4.37011e-70),
                 padj = c(1.49936e-35, 1.12976e-67),
                 row.names = c("DLK1", "EPCAM"))
# input list
ind_deg <- list(DEG1_df = DGE1, DEG2_df = DGE2)
# perform invnorm meta-analysis
invnorm <- metaRNAseq(ind_deg, test_statistic = "invnorm", BHth = 0.05, nrep = c(2,2))
# perform fishercomb meta-analysis
fishercomb <- metaRNAseq(ind_deg, test_statistic = "fishercomb", BHth = 0.05)
# combine results
comb_pval_df <- combine_fisher_invnorm(ind_deg,
                                      invnorm, fishercomb,
                                      adjpval = 0.05,
                                      output_tsv = FALSE)}
}
\seealso{
\code{\link{DGE}} function for DGE analysis,
and \url{https://cran.r-project.org/web/packages/metaRNASeq/vignettes/metaRNASeq.pdf}
for metaRNASeq package info

Other meta-analysis functions: 
\code{\link{metaRNAseq}()}
}
\concept{meta-analysis functions}

\name{ind_deg}
\alias{ind_deg}
\title{ind_deg}
\value{
A list of dataframes.
}
\description{
  Input list for metaRNAseq function made of 2 different small Deseq2
   output samples dataframes for testing purposes: DEG1_df and DEG2_df.
}
\usage{
  data(ind_deg)
}
\format{
  dataframe list
}
\details{
  Each dataframe has 2 rows and 9 columns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment_barplot.R
\name{Enrichment_barplot}
\alias{Enrichment_barplot}
\title{Enrichment_barplot}
\usage{
Enrichment_barplot(
  Enrich,
  enrich.databases = c("GO_Biological_Process_2021", "GO_Cellular_Component_2021",
    "GO_Molecular_Function_2021"),
  p_adj = 0.05,
  num_term = 10,
  cond = "UP",
  plot = FALSE
)
}
\arguments{
\item{Enrich}{A list of enrichment tables for up and down-regulated genes in the different enrichR databases.
Output of Enrichment.R function for one DGE experiment.}

\item{enrich.databases}{Vector of EnrichR databases to consider. These databases must be present in the Enrich list.}

\item{p_adj}{Double. Minimum Adjusted pvalue threshold for the enrichment}

\item{num_term}{Double. Number of up-regulated and dw-regulated terms to represent}

\item{cond}{String. Title of the plot.}

\item{plot}{Logical. If TRUE save plot as pdf.}
}
\value{
bar plot of significant pathways.
}
\description{
Barplot representing the top up-regulated or down-regulated significant pathways
}
\examples{
\dontrun{
dbs <- c("GO_Biological_Process_2021")
dfList <- list()
dfList[["fdr_up"]]$GO_Biological_Process_2021 <- data.frame(
  Term = c("peripheral nervous system neuron differentiation (GO:0048934)",
           "apoptotic chromosome condensation (GO:0030263)",
           "negative regulation of CD4-positive, alpha-beta T cell differentiation (GO:0043371)"),
  Overlap = c("1/5", "1/5", "1/5"),
  P.value = c(0.0007498315, 0.0007498315, 0.0007498315),
  Adjusted.P.value = c(0.00893491, 0.00893491, 0.00893491),
  Old.P.value = c(0, 0, 0),
  Old.Adjusted.P.value = c(0, 0, 0),
  Odds.Ratio = c(2499.125, 2499.125, 2499.125),
  Combined.Score = c(17982.86, 17982.86, 17982.86),
  Genes = c("RUNX1", "TOP2A", "RUNX1"))
dfList[["fdr_down"]]$GO_Biological_Process_2021 <- data.frame(
  Term = c("skin morphogenesis (GO:0043589)",
           "skin development (GO:0043588)",
           "collagen fibril organization (GO:0030199)"),
  Overlap = c("2/7", "2/80", "2/89"),
  P.value = c(3.149296e-07, 4.727687e-05, 5.856991e-05),
  Adjusted.P.value = c(1.291211e-05, 8.004554e-04, 8.004554e-04),
  Old.P.value = c(0, 0, 0),
  Old.Adjusted.P.value = c(0, 0, 0),
  Odds.Ratio = c(7996.8000, 510.7436, 457.7011),
  Combined.Score = c(119719.427, 5086.745, 4460.430),
  Genes = c("COL1A1;COL1A2", "COL1A1;COL1A2", "COL1A1;COL1A2"))
Enrichment_barplot(dfList,
                   enrich.databases = dbs,
                   p_adj = 0.01, num_term = 3, cond = "UP")
}
}
\seealso{
Other functional-annotation functions: 
\code{\link{Annotate_SPID}()},
\code{\link{Enrichment}()}

Other plot functions: 
\code{\link{SVenn}()},
\code{\link{Splot}()},
\code{\link{plotPCA}()}
}
\concept{functional-annotation functions}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Enrichment.R
\name{enrichr}
\alias{enrichr}
\title{Gene enrichment using Enrichr}
\usage{
enrichr(genes, databases)
}
\arguments{
\item{genes}{(Required). Character vector of Entrez gene symbols as input. A data.frame
of gene symbols in first column is also acceptable, optionally a score denoting the
degree of membership between 0 and 1 in the second column.}

\item{databases}{(Required). Character vector of databases to search.
See https://maayanlab.cloud/Enrichr/ for available databases.}
}
\value{
Returns a list of data.frame of enrichment terms, p-values, ...
}
\description{
Gene enrichment using Enrichr
}
\details{
Gene enrichment using Enrichr, slighthly modified by Aurora Maurizio.
}
\examples{
\dontrun{
GeneID  = c("MEST", "CDK1", "PCLAF", "BIRC5")
dbs <- c("GO_Molecular_Function_2023", "GO_Cellular_Component_2023",
         "GO_Biological_Process_2023")
enriched1 <- enrichr(GeneID, dbs)
print(head(enriched1[[1]]))}
}
\author{
Wajid Jawaid \email{wj241@alumni.cam.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOmetadata.R
\name{GEOmetadata}
\alias{GEOmetadata}
\title{GEOmetadata function}
\usage{
GEOmetadata(GSE, GPL = "")
}
\arguments{
\item{GSE}{The GSE series ID.}

\item{GPL}{The GPL series numbers.
Required only if the chosen GSE series ID include data from multiple sequencing platforms.}
}
\value{
A dataframe with all the available characteristics in GEO metadata \code{genes} array.
}
\description{
Download metadata from \url{https://www.ncbi.nlm.nih.gov/geo},
given an input GEO accession series.
}
\section{Warning}{

If the GEO accession series has more than 1 sequencing platforms you need to
specify the GPL series numbers.
}

\examples{
# only one sequencing platform
\dontrun{
mGSE133671 <- GEOmetadata(GSE = "GSE133671")
# multiple sequencing platforms
mGSE59483 <- GEOmetadata("GSE59483", GPL = c("GPL11154", "GPL15520"))}
}
\seealso{
\url{https://www.ncbi.nlm.nih.gov/geo} for info on GEO repository

Other public-data functions: 
\code{\link{DownloadArchS4}()},
\code{\link{TCGA_download}()}
}
\concept{public-data functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPCA.R
\name{plotPCA}
\alias{plotPCA}
\title{plotPCA function}
\usage{
plotPCA(
  matrix,
  metadata,
  nTOP = 500,
  dims = c(1, 2),
  centering = TRUE,
  scaling = TRUE,
  color.by = NULL,
  shape.by = NULL,
  pt.size = 6,
  cols.use = NULL,
  shape.use = NULL,
  main = "PCA",
  label = FALSE,
  new.label = NULL
)
}
\arguments{
\item{matrix}{Filtered count matrix in CPM or RPKM with gene on the row and sample ID on the column.}

\item{metadata}{Sample metadata, row.names must be samples names.}

\item{nTOP}{number of top genes to use for principal components,
selected by highest row variance}

\item{dims}{Dimensions to plot, must be a two-length numeric vector specifying x- and y-dimensions}

\item{centering}{Logical. If \code{TRUE} center PCs}

\item{scaling}{Logical. If \code{TRUE} scales PCs}

\item{color.by}{Name of one or more metadata columns to color point by.}

\item{shape.by}{Name of one or more metadata columns to shape point by. If NULL, all points are circles \(default\).}

\item{pt.size}{Size of the points in the plot.}

\item{cols.use}{Vector of colors, each color corresponds to an identity class.
By default, ggplot assigns colors.}

\item{shape.use}{Vector of shape, each shape corresponds to an identity class.}

\item{main}{Plot title. Default = PCA.}

\item{label}{Logical. If \code{TRUE} adds samples label. Default = FALSE.}

\item{new.label}{If NULL, use the sample names as in metadata row.names.
Otherwise you can specify new labels.}
}
\value{
PCA plot objec created by ggplot2,
which can be assigned and further customized.
}
\description{
Plot PCA highlighting one or two data features
}
\examples{
\dontrun{
# Simulation of bulk RNA data
countData <- matrix(floor(runif(10000, min=0, max=101)),ncol=4)
colnames(countData) <- paste("sample", seq_len(ncol(countData)), sep = "")
rownames(countData) <- paste("gene", seq_along(seq_len(10000/4)), sep = "")
metadata <- data.frame(samplesID = paste("sample", seq_len(ncol(countData)), sep = ""),
                     condition = factor(c("A","A","B","B")),
                     therapy = factor(c("T1","T2","T1","T2")))
row.names(metadata) <- metadata$samplesID
library(edgeR)
SurfR::plotPCA(matrix = cpm(countData),
        metadata = metadata,
        nTOP = 100,
        dims = c(1,2),
        color.by = "condition", shape.by = "therapy",
        label = FALSE, main = "PCA")}
}
\seealso{
Other plot functions: 
\code{\link{Enrichment_barplot}()},
\code{\link{SVenn}()},
\code{\link{Splot}()}
}
\concept{plot functions}
